
#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include "frame.h"
#include "libs.h"

extern FILE* debug_file;
#define LIST_INCREMENT 10

int item_cmp(const void* p1, const void* p2)
{
	return( ( ((DirtyItem*) p1)->y) -  ( ((DirtyItem*) p2)->y) );

}

void erase_pixel(DirtyItem *item, BITMAP *frame)
{
  putpixel(frame, item->x, item->y, pallete_color[0]);
}

void draw_pixel(DirtyItem *item, BITMAP *frame, BITMAP *child)
{
  putpixel(child, item->x, item->y, getpixel(frame, item->x, item->y));
}

void erase_box(DirtyItem *item, BITMAP *frame)
{
    rectfill(frame, item->x,
										item->y,
      							item->x + item->a,
										item->y + item->b, pallete_color[0]);
}

void draw_box(DirtyItem *item, BITMAP *frame, BITMAP *child)
{
//  if((item->a == 1) && (item->b == 1))
//    putpixel(child, item->x, item->y, getpixel(frame, item->x, item->y));
//  else

    blit(frame, child, item->x, 
												item->y, 
												item->x, 
												item->y, 
												item->a, 
												item->b);
}

void erase_line(DirtyItem *item, BITMAP *frame)
{
  if((item->x == item->a) && (item->y == item->b))
    putpixel(frame, item->x, item->y, pallete_color[0]);
  else
    line(frame, item->x, item->y, item->a, item->b, pallete_color[0]);
}

BITMAP *line_frame;

void line_pixel(BITMAP *line_child, int x, int y, int d)
{
  putpixel(line_child, x, y, getpixel(line_frame, x, y));
}

void draw_line(DirtyItem *item, BITMAP *frame, BITMAP *child)
{
  if((item->x == item->a) && (item->y == item->b))
    putpixel(child, item->x, item->y, getpixel(frame, item->x, item->y));
  else {
    line_frame = frame;
    do_line(child, item->x, item->y, item->a, item->b, 0, line_pixel);
  }
}

Frame::Frame(int x, int y, int w, int h, int max_items) {
  acquire_screen();
  child = create_sub_bitmap(screen, x, y, w, h);
  frame = create_bitmap(w, h);
  release_screen();
	clear_to_color(frame, palette_color[0]);

  list_size = max_items;

  item_count = 0;
  item       = new DirtyItem[list_size];


  old_item_count = 0;

  old_item       = new DirtyItem[list_size];
}

Frame::~Frame() {
  
#ifdef SCP_DEBUG	  
	fprintf(debug_file, "- Frame -> Destructor begins\n");
  fflush(debug_file);
  #endif
		
	destroy_bitmap(frame);

  delete item;
  item_count = 0;

  delete old_item;
  old_item_count = 0;
}

void Frame::enlarge_list(int increment)
{
  
  #ifdef SCP_DEBUG  
  fprintf(debug_file, "- Frame -> enlarge_list()\n");
  fflush(debug_file);
  #endif
		
  // Static so no deleties
  static DirtyItem *temp;

  temp = item;
  item = new DirtyItem[list_size + increment];
  memcpy(item, temp, item_count * sizeof(DirtyItem));

  temp = old_item;
  old_item = new DirtyItem[list_size + increment];
  memcpy(old_item, temp, old_item_count * sizeof(DirtyItem));

  temp = NULL;
  
  list_size += increment;
}

void Frame::add_to_list(int x, int y, int a, int b,
  void (*erase_item)(DirtyItem *item, BITMAP *frame),
  void (*draw_item)(DirtyItem *item, BITMAP *frame, BITMAP *child))
{
  if(item_count == list_size)
    enlarge_list(LIST_INCREMENT);

  item[item_count].x = x;
  item[item_count].y = y;
  item[item_count].a = a;
  item[item_count].b = b;

  item[item_count].erase_item = erase_item;
  item[item_count].draw_item  = draw_item;

  item_count++;
}

void Frame::add_to_old_list(int x, int y, int a, int b,
  void (*erase_item)(DirtyItem *item, BITMAP *frame),
  void (*draw_item)(DirtyItem *item, BITMAP *frame, BITMAP *child))
{
  if(old_item_count == list_size)
    enlarge_list(LIST_INCREMENT);

  old_item[old_item_count].x = x;
  old_item[old_item_count].y = y;
  old_item[old_item_count].a = a;
  old_item[old_item_count].b = b;

  old_item[old_item_count].erase_item = erase_item;
  old_item[old_item_count].draw_item  = draw_item;

  old_item_count++;
}

 void Frame::add_pixel(int x, int y, int a, int b)
{
  add_to_list(x, y, a, b, erase_pixel, draw_pixel);
}

 void Frame::add_box(int x, int y, int a, int b)
{
//  add_to_list(x, y, a, b, erase_box, draw_box);
  add_to_list(x - OVERSIZE_BOXES, 
							y - OVERSIZE_BOXES, 
							a + OVERSIZE_BOXES * 2, 
							b + OVERSIZE_BOXES * 2, erase_box, draw_box);
}

 void Frame::add_line(int x, int y, int a, int b)
{
  add_to_list(x, y, a, b, erase_line, draw_line);
}

 void Frame::add_old_pixel(int x, int y, int a, int b)
{
  add_to_old_list(x, y, a, b, erase_pixel, draw_pixel);
}

 void Frame::add_old_box(int x, int y, int a, int b)
{
  add_to_old_list(x, 
									y, 
									a, 
									b, erase_box, draw_box);
}

 void Frame::add_old_line(int x, int y, int a, int b)
{
  add_to_old_list(x, y, a, b, erase_line, draw_line);
}

void Frame::erase()
{
  int c;
 
  for(c = 0; c < item_count; c++) {
    item[c].erase_item(&item[c], frame);
  }

  memcpy(old_item, item, item_count * sizeof(DirtyItem));
  old_item_count = item_count;
  item_count = 0;
}

void Frame::draw()
{
  int c;

  for(c = 0; c < item_count; c++)
    add_to_old_list(item[c].x, item[c].y, item[c].a, item[c].b,
      item[c].erase_item, item[c].draw_item);

  if(!gfx_driver->linear)
    qsort(old_item, old_item_count, sizeof(DirtyItem), item_cmp);

  for(c = 0; c < old_item_count; c++)
    old_item[c].draw_item(&old_item[c], frame, child);
}
