#ifndef _MELEE_H
#define _MELEE_H

//#define SCP_DEBUG

#include <stdio.h>
#include <allegro.h>
#include <math.h>
#ifndef PI
  #define PI 3.1415926535
#endif

#ifdef _WIN32
  #include <winalleg.h>
  #define random()  rand()
  #define srandom(a) srand(a)
#endif

struct JGMOD;

#define MELEE_ASTEROID          0
#define MELEE_ASTEROIDEXPLOSION 64
#define MELEE_BOOM              84
#define MELEE_BOOMSHIP          88
#define MELEE_HOTSPOT           89
#define MELEE_KABOOM            101
#define MELEE_MUSIC             121
#define MELEE_PANEL             122
#define MELEE_PLANET            125
#define MELEE_SPARK             128
#define MELEE_STAR              134

#define SCPPAL_PALLETE 0
#define SCPPAL_FONT    4

#define PHASE_MAX   12
#define PHASE_DELAY 4

#define X_MAX 4096.0
#define Y_MAX 3840.0

#define VIEW_X 512
#define VIEW_Y 480

#define MAX_SHIPS      8
#define MAX_STARS      100
#define MAX_ASTEROIDS  8
#define MAX_HOTSPOTS   20

#define SPACE_LAYERS 6

#define LAYER_HOTSPOTS   0
#define LAYER_CBODIES    1
#define LAYER_SHIPS      2
#define LAYER_LINES      3
#define LAYER_SHOTS      4
#define LAYER_EXPLOSIONS 5

#define MAX_SPACE_ITEMS (2 * (1 + MAX_ASTEROIDS + (MAX_SHIPS * (32 + MAX_HOTSPOTS)) + (MAX_STARS * 3)))
#define MAX_RADAR_ITEMS (2 * (1 + MAX_ASTEROIDS + MAX_SHIPS))

#define MAX_FLEET_SIZE 50
#define EMPTY_SLOT     -1

#define KEYFLAG_LEFT     1
#define KEYFLAG_RIGHT    2
#define KEYFLAG_THRUST   4
#define KEYFLAG_FIRE     8
#define KEYFLAG_SPECIAL  16
#define KEYFLAG_NEXT     32
#define KEYFLAG_PREV     64
#define KEYFLAG_CLOSE    128

extern FILE *debug_file;

extern char *key_string[];
int key_num (const char *name);

//extern PPCOL_MASK *pix;
extern double space_x;
extern double space_y;
extern double space_zoom;
/*
extern int melee_rate;*/

extern FONT *desktop_font;
extern void (*tw_draw_rle_sprite)(BITMAP *bmp, RLE_SPRITE *sprite, int x, int y);

#define CAPTAIN_WIDTH   55
#define CAPTAIN_HEIGHT  30

#define ASTEROID_FRAMES          64
#define ASTEROIDEXPLOSION_FRAMES 20
#define HOTSPOT_FRAMES           12
#define KABOOM_FRAMES            20
#define SPARK_FRAMES             6

#define RADAR_COLOR_FRIEND   2
#define RADAR_COLOR_ENEMY    4
#define RADAR_COLOR_PLANET   9
#define RADAR_COLOR_ASTEROID 8

#define FROM_ABOVE 0
#define FROM_BELOW 1
#define FROM_LEFT  2
#define FROM_RIGHT 3

#define bit(a) (1 << (a))

#define MAX_SAMPLES  8
#define BOOM_SAMPLES 4

#define ANGLE_RATIO     (3.1415926535 / 180.0)
#define RANGE_RATIO     40.0
#define VELOCITY_RATIO  7.5
#define ACCELRATE_RATIO 100.0

#define HOT_COLORS 12

extern int hot_color[HOT_COLORS];

//frame.h
struct DirtyItem;
class Frame;

//ppcol.h
struct PPCOL_MASK;

//melee/mmain.h
extern DATAFILE *melee;
extern int debug_value;

//melee/mcontrol.h
class Control;

//melee/mview.h
class View;

//
class Shot;
class AnimatedShot;
class Missile;
class HomingMissile;
class Laser;

//
class SpaceFrame;
class SpaceSprite;
class SpaceLocation;
class SpaceObject;
class SpaceLine;

//
class Animation;
class Asteroid;
class Planet;
class Ship;
class ShipData;
class ShipPanel;
class Stars;

//
extern SpaceFrame *spaceFrame;

//
extern SpaceSprite *asteroidSprite;
extern SpaceSprite *asteroidExplosionSprite;
extern SpaceSprite *hotspotSprite;
extern SpaceSprite *kaboomSprite;
extern SpaceSprite *sparkSprite;

extern JGMOD *meleeVictory;

#define MAX_SHIP_TYPES 100


class SpaceSprite {
	protected:
	int         count;
	int         w;
	int         h;
	PPCOL_MASK **m;
	BITMAP     **b;

	public:
	SpaceSprite(DATAFILE *sprites, int sprite_count);
	SpaceSprite(BITMAP **bmp, int bmp_count);

	virtual ~SpaceSprite();

	BITMAP     *get_bitmap(int index);

	virtual void animate(int x, int y, int index, Frame *space);

	void draw_character(int x, int y, int index, int color, BITMAP *bmp);
	void draw_character_stretch(int x, int y, int index, int color, double zoom, BITMAP *bmp);
	void animate_character(int x, int y, int index, int color, Frame *space);

	int collide(int x, int y, int i, int ox, int oy, int oi,
			SpaceSprite *other);
	int collide_line(int lx1, int ly1, int *lx2, int *ly2, int sx, int sy,
			int sindex);

	int    frames();
	double width();
	double height();
	};

class SpaceRotatingSprite : public SpaceSprite {
  public:
  SpaceRotatingSprite(DATAFILE *sprites, int sprite_count);
};
/*
// Used in AI scripts -DF
typedef struct {
  double normal_x;
  double normal_y;
  double vx;
  double vy;
  double angle;
  int id;
  int batt;
  int crew;
} SC_SPACEOBJECT;
*/

typedef struct {
  char *id;
  char *name;
  Ship *(*getShip)(double sX, double sY, double sAngle, int sCollideFlag,
    int sRadarColor, ShipData *sData, Control *cntrl);
} SHIP_REGISTER_INFO;

typedef struct {
  SHIP_REGISTER_INFO *reg;
  ShipData           *dat;
  int                cost;
} SHIP_INFO;

void register_ship(SHIP_REGISTER_INFO *reg_info);
void init_ships();

void tw_error(char* message);

char *shipListboxGetter(int index, int *list_size);
char *fleetListboxGetter(int index, int *list_size);

double normalize(double value, double max);
double wcorner(double x, double w);
double hcorner(double y, double h);
double min_delta(double from, double to, double max);
double nearest_coord(double from, double to, double max);
double distance_from(double x, double y, double to_x, double to_y);
void   scale_frame_rate(int *value);

extern void (*tw_draw_rle_sprite)(BITMAP *bmp, RLE_SPRITE *sprite, int x, int y);
void tw_draw_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y);

int tw_sample (SAMPLE *spl, int vol = 256, int pan = 128, int freq = 1000) ;

class AI_Player;

class SpaceLocation {
  protected:
  int    id;
  double x;
  double y;
  double angle;
  double vx;
  double vy;
  double driftvx;
  double driftvy;
  public:
  int    state;
	int    layer;
  public:
  int    damage_factor;
  int    collide_flag_anyone;
  int    collide_flag_ally;
  int    ally_flag;

  friend class Planet;

  public:
  int    damage;
  int    directDamage;
  int    repair;
  int    fuel_sap;
  double speed_loss;

  SpaceLocation(double lx, double ly, double langle, int team);
	
  SpaceLocation *newLocation(double lx, double ly, double langle);

  double get_angle();
  double get_vx();
  double get_vy();

  int getID();
//  int collideFlag();
  int allyFlag();

  double normal_x();
  double normal_y();
  double nearest_x(SpaceLocation *l);
  double nearest_y(SpaceLocation *l);
  double distance(SpaceLocation *l);
  double trajectory_angle(SpaceLocation *l);

  virtual int exists();
  virtual int canCollide(SpaceLocation *other);
  virtual int isAlly(SpaceLocation *other);

	virtual void translate(double dx, double dy);
};

class SpaceObject : public SpaceLocation {
  public:
  double       w;
  double       h;
  double       mass;
  protected:
  SpaceSprite *sprite;
  int          sprite_index;
  int          radar_color;

  friend class Planet;
  friend class SpaceLine;

  public:
  SpaceObject *next;

  SpaceObject(double ox, double oy, double oangle, int ocflag,
    SpaceSprite *osprite);

  double corner_x();
  double corner_y();

  SpaceSprite *getSprite();

  int getSpriteIndex();

  double width();
  double height();

  virtual void animate(Frame *space, Frame *radar);
  virtual void collide(SpaceObject *other);
  virtual double collide_line(double lx1, double ly1, double lx2, double ly2,
    double llength);

  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage();

  virtual void calculate();

  virtual void pull(double ang, double force);
  void pull(SpaceObject *other, double pullVelocity);
};

class SpaceLine : public SpaceLocation {
  protected:
  double length;
  int    color;

  SpaceObject *collide_object;

  public:
  SpaceLine *next;

  SpaceLine(double lx, double ly, double langle, int lcflag, double llength,
    int lcolor);

//  virtual int canCollide(SpaceLocation *other);

  double edge_x();
  double edge_y();
  double get_length();

  virtual void calculate();
  virtual void animate(Frame *space);
  virtual void collide(SpaceObject *o);
};

class SpaceFrame {
  private:
  SpaceObject **firstLayerObject;
  SpaceLine    *firstLine;
  Frame        *space;
  Frame        *radar;
  int           numLayers;

  public:
  SpaceFrame(Frame *spaceFrame, Frame *radarFrame, int layerCount);
  ~SpaceFrame();

  SpaceObject *addSpaceObject(SpaceObject *o);
  SpaceObject *getFirstObject(int layer);

  SpaceLine *addSpaceLine(SpaceLine *l);
  SpaceLine *getFirstLine();

  void calculate();
  void collide();
  void animate();
};

class Animation : public SpaceObject {
  protected:
  int frame_count;
  int frame_size;
  int frame_step;

  public:
  Animation(double ox, double oy, SpaceSprite *osprite, int ofcount,
    int ofsize, int layer);

  virtual void calculate();
};

class FixedAnimation : public Animation {
  SpaceLocation *pos;

  public:
  FixedAnimation(SpaceLocation *opos, SpaceSprite *osprite, int ofcount,
    int ofsize, int olayer);

  virtual void calculate();
};

class ShipData {
  public:
  DATAFILE    *data;
  BITMAP     **bitmapPanel;
  SpaceSprite *spriteShip;
  SpaceSprite *spriteWeapon;
  SpaceSprite *spriteWeaponExplosion;
  SpaceSprite *spriteSpecial;
  SpaceSprite *spriteSpecialExplosion;
  SpaceSprite *spriteExtra;
  SpaceSprite *spriteExtraExplosion;
  SAMPLE     **sampleWeapon;
  SAMPLE     **sampleSpecial;
  SAMPLE     **sampleExtra;
  JGMOD       *moduleVictory;

  ShipData(char *filename);
  ~ShipData();
};

class ShipPanel {
  protected:
  BITMAP *panel;
  BITMAP *captain;

  int panel_x;
  int panel_y;
  int captain_x;
  int captain_y;
  int captain_animated;
  int crew_old;
  int batt_old;
  int batt_light;

  int old_thrust;
  int old_turn_left;
  int old_turn_right;
  int old_fire_weapon;
  int old_fire_special;

  int fryFrame;

  public:
  Ship *ship;
  Ship *nextShip;

  ShipPanel(Ship *oship, int oindex);
  ~ShipPanel();

  int exists();

  void refresh(int index);
  void initialize(Ship *oship, int shipIndex);

  void calculate();
  void animate();
};

class keyflag {
  public:
  static const int left   ;
  static const int right  ;
  static const int thrust ;
  static const int fire   ;
  static const int special;
  static const int next;
  static const int prev;
  static const int close;
};

class Ship : public SpaceObject {
  protected:
  SpaceSprite *phaseSprite;

  int hotspot_frame;
  int phase_count;
  int phase_step;
  int recharge_step;
  int weapon_recharge;
  int weapon_low;
  int special_recharge;
  int special_low;

  void calculate_turn_rate();
  void calculate_speed_max();
  void calculate_accel_rate();

  virtual void materialize();

  virtual void calculate_phase();
  virtual void calculate_thrust();
  virtual void calculate_turn_left();
  virtual void calculate_turn_right();
  virtual void calculate_fire_weapon();
  virtual void calculate_fire_special();
  virtual void calculate_gravity();
  virtual void calculate_hotspots();

  virtual int activate_weapon();
  virtual int activate_special();

  double target_x;
  double target_y;

  public:
  int    crew;
  int    crew_max;
  int    batt;
  int    batt_max;
  double turn_rate;
  double turn_step;
  double speed_max;
  double accel_rate;
  int    recharge_amount;
  int    recharge_rate;
  int    weapon_drain;
  int    weapon_rate;
  int    weapon_sample;
  int    special_drain;
  int    special_rate;
  int    special_sample;
  int    hotspot_rate;

  BITMAP   *panelBitmap[7];
  ShipData *data;
  Ship     *target;

  int update_panel;

  int nextkeys;

  int thrust;
  int turn_left;
  int turn_right;
  int fire_weapon;
  int fire_special;
	int target_next;
	int target_prev;
	int target_close;

	int target_pressed;
	Control *control;

  friend class ShipPanel;

  Ship(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);
  virtual ~Ship();

  virtual int getCrew();
  virtual int getBatt();
  virtual int crewPanelColor();
  virtual int battPanelColor();

  void locate();
  
  virtual int isCrewRobots() { return FALSE; }

  void calculate();

  void animate(Frame *space, Frame *radar);

  void handle_damage();
};

class Shot : public SpaceObject {
  protected:
  Ship  *ship;
  double v;
  double d;
  double range;
  int    armour;
  
  SpaceSprite *explosionSprite;
  SAMPLE      *explosionSample;
  int          explosionFrameCount;
  int          explosionFrameSize;

  public:
  Shot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceLocation *opos,
    SpaceSprite *osprite);

  virtual void calculate();
  virtual void handle_damage();
  virtual void inflict_damage(SpaceObject *other);

  virtual void animateExplosion();
  virtual void soundExplosion();

  void stop();
  void destroy();
};

class AnimatedShot : public Shot {
  protected:
  int frame_count;
  int frame_size;
  int frame_step;

  public:
  AnimatedShot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceLocation *opos,
    SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
};

class Missile : public Shot {
  public:
  Missile(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceLocation *opos,
    SpaceSprite *osprite);
};

class HomingMissile : public Missile {
  protected:
  double turn_rate;
  double turn_step;

  public:
  HomingMissile(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);

  virtual void calculate();
};

class Laser : public SpaceLine {
  protected:
  int frame;
  int frame_count;

  SpaceLocation *pos;
  double pos_x;
  double pos_y;

  public:
  Laser(double langle, int lcolor, double lrange, int ldamage, int lfcount,
    SpaceLocation *opos, double rel_x, double rel_y);

  void calculate();
};

#endif
