
#include <stdlib.h>

#include "../melee.h"
#include "../id.h"

#include "mview.h"
#include "mcbodies.h"
#include "mmain.h"

// Define global planet object
Planet *spacePlanet;

// Define global asteroid sprites
SpaceSprite *asteroidSprite;
SpaceSprite *asteroidExplosionSprite;

Stars::Stars()
{
  int i, j;

  for(i = 0; i < 3; i++) {
    sd[i].s = new SpaceSprite(&melee[MELEE_STAR + i], 1);
    for(j = 0; j < MAX_STARS; j++) {
      sd[i].x[j] = (double)(random() % (int)(X_MAX * pow(2.0, (double)(i))));
      sd[i].y[j] = (double)(random() % (int)(Y_MAX * pow(2.0, (double)(i))));
    }
  }
}

Stars::~Stars()
{
	#ifdef SCP_DEBUG
  fprintf(debug_file, "- Stars -> Destructor\n");
	fflush(debug_file);
  #endif
	int i;

  for(i = 0; i < 3; i++)
    delete sd[i].s;

	#ifdef SCP_DEBUG
  fprintf(debug_file, "- Stars -> Destroyed successfully\n");
	fflush(debug_file);
  #endif
}

//MYCODE

void Stars::animate(Frame *space)
{
  int i, j;
  double sx, sy, sw, sh, sz;

	double pow2;
  for(i = 2; i >= 0; i -= 1) {
		pow2 = pow(2.0, -i);
    sw = sd[i].s->width();
    sh = sd[i].s->height();
    sz = space_zoom;
    for(j = 0; j < MAX_STARS; j++) {
      sx = sd[i].x[j] * sz;
      sx -= space_x_stars * pow2 * sz;
      sy = sd[i].y[j] * sz;
      sy -= space_y_stars * pow2 * sz;

      while(sx < -X_MAX * sz / 2)
        sx +=     X_MAX * sz;
      while(sx >  X_MAX * sz / 2)
        sx -=     X_MAX * sz;
      while(sy < -Y_MAX * sz / 2)
        sy +=     Y_MAX * sz;
      while(sy >  Y_MAX * sz / 2)
        sy -=     Y_MAX * sz;


      sd[i].s->animate(sx + VIEW_X / 2, sy + VIEW_Y / 2, 0, space);
    }
	pow2 /= 2;
//		if (zoom_discrete) pow2/=2.0; //when zooming is continous, all stars are "flattened"
	}
}

Asteroid::Asteroid() :
  SpaceObject(0.0, 0.0, 0.0, -1, asteroidSprite)
{
  collide_flag_anyone = 0xff;
	collide_flag_ally = 0xff;
	layer = LAYER_CBODIES;
  id    = SPACE_ASTEROID;
  step  = 0;
  speed = (random() % 3) + 1;
  mass  = 10 + (random() % 10);

  radar_color = RADAR_COLOR_ASTEROID;

  // Angles are clockwise, turned left at zero degrees
  switch(random() % 4) {
    case FROM_ABOVE:
      // Entering from upper portion
      x = (double)(random() % (int)(X_MAX));
      y = 0.0;

      // Between 45 to 135 degrees
      angle = (double)(random() % 90) + 45.0;
      break;

    case FROM_BELOW:
      // Entering from lower portion
      x = (double)(random() % (int)(X_MAX));
      y = Y_MAX;

      // Between 225 to 315 degrees
      angle = (double)(random() % 90) + 225.0;
      break;

    case FROM_LEFT:
      // Entering from left portion
      x = 0.0;
      y = (double)(random() % (int)(Y_MAX));

      // Between -45 to 45 degrees
      angle = (double)(random() % 90) - 45.0;
      break;

    case FROM_RIGHT:
      // Entering from right portion
      x = X_MAX;
      y = (double)(random() % (int)(Y_MAX));

      // Between 125 to 215 degrees
      angle = (double)(random() % 90) + 125.0;
      break;
  }

  while(angle < 0.0)
    angle += 360.0;
  while(angle > 360.0)
    angle -= 360.0;

  vx = speed * cos(angle * ANGLE_RATIO);
  vy = speed * sin(angle * ANGLE_RATIO);

  speed *= 5;
}

void Asteroid::calculate()
{
  if(step == speed) {
    step = 0;
    sprite_index++;
    if(sprite_index == ASTEROID_FRAMES)
      sprite_index = 0;
  } else
    step++;

  SpaceObject::calculate();
}

void Asteroid::handle_damage()
{
  if((damage == 0) && (directDamage == 0))
    return;

  damage = 0;
  directDamage = 0;
  spaceFrame->addSpaceObject(new Animation(x, y,
    asteroidExplosionSprite, ASTEROIDEXPLOSION_FRAMES, 5, LAYER_CBODIES));

  do {
    x = (double)(random() % (int)(X_MAX));
    y = (double)(random() % (int)(Y_MAX));
  } while(
    (fabs(min_delta(x, normalize(space_x + (X_MAX / 4.0), X_MAX), X_MAX)) < (X_MAX / 4.0)) ||
    (fabs(min_delta(y, normalize(space_y + (Y_MAX / 4.0), Y_MAX), Y_MAX)) < (Y_MAX / 4.0)));

  while(angle < 0.0)
    angle += 360.0;
  while(angle > 360.0)
    angle -= 360.0;
}

Planet::Planet(SpaceSprite *planetSprite) :
  SpaceObject(X_MAX / 2.0, Y_MAX / 2.0, 0.0, -1, planetSprite)
{
  collide_flag_anyone = 0xff;
	collide_flag_ally = 0xff;
	layer = LAYER_CBODIES;
  id          = SPACE_PLANET;
  mass        = 99999.0;
  radar_color = RADAR_COLOR_PLANET;
}

void Planet::inflict_damage(SpaceObject *other)
{
  int i = 1;

  if (other->getID() == SPACE_SHIP) {
    i = 1 + ((Ship*)other)->crew / 3;
    }    
  other->directDamage += i;
	i >>= 1;
  if (i >= BOOM_SAMPLES)
    i = BOOM_SAMPLES - 1;
	tw_sample((SAMPLE *)(melee[MELEE_BOOM + i].dat));

// Velocity based damage system

/*  if(other->mass >= 0.0) {
    i = (int)(fabs(sqrt(
      ((other->vx + other->driftvx) * (other->vx + other->driftvx)) +
      ((other->vy + other->driftvy) * (other->vy + other->driftvy))) *
      other->mass) / 32.0);
    if(i >= BOOM_SAMPLES)
      i = BOOM_SAMPLES - 1;

		tw_sample((SAMPLE *)(melee[MELEE_BOOM + i].dat));

    switch(i) {
      case 0:
        other->directDamage += 1; break;
      case 1:
        other->directDamage += 3; break;
      case 2:
        other->directDamage += 6; break;
      case 3:
        other->directDamage += 12; break;
    }
  };*/

	other->handle_damage();
	}

