#ifndef _MELEE_H
#include "../melee.h"
#endif

#ifndef _MCONTROL_H
#define _MCONTROL_H

class Control {	
	public:
  Ship *ship;
//points at the current ship being controlled
	char *iname;
//name of instance
	int num_targets;
//number of targets
	Ship **target;
//array of targets
	int active_target;
  public:
  virtual void load(char* inifile, char* inisection);
//loads configuration data from a ini file. 
//example:  player1.load("scp.ini", "Keyboard1");
  virtual void save(char* inifile, char* inisection);
//saves configuration data to a ini file. 
  virtual void select_ship(Ship* ship_pointer, char* ship_name);
//called whenever the ship being controlled changes
  void calculate();
//called every frame of physics
  virtual char think();
//called by calculate
  virtual void initialize() ;
//called whenever a new melee is started
  virtual void deinitialize();
//called whenever the current melee ends
  virtual char *getTypeName() = 0;
//returns the name of the control type, like "Joystick"
  virtual char *getDescription();
//returns the description of the control, like "NetworkGame(local)_Keyboard"
	Control (char *name) ;
//the constructor initializes ship to NULL
	virtual void setup();
//called by the GUI stuff to setup the controls (calibrate Joystick, set keys, etc.)
	virtual void add_target(Ship *killit) ;
	virtual void remove_target(Ship *killit) ;
	virtual void set_target(int i) ;
//sets the target
	};

class ControlKB : public Control {	
	private:
	int thrust;
	int left;
	int right;
	int fire;
	int special;
	int next;
	int prev;
	int close;
  public:
  virtual void load(char* inifile, char* inisection);
  virtual void save(char* inifile, char* inisection);
  virtual char *getTypeName();
  virtual char think();
	ControlKB (char *name) ;
	virtual void setup();
	};

class ControlMoron : public Control {	
  public:
  virtual char *getTypeName();
  virtual char think();
	ControlMoron (char *name) ;
	};

#define SPECIAL_UNKNOWN 0
#define SPECIAL_DEFENSE 1
#define SPECIAL_BATTERY 2
class ControlWussie : public Control {
  public:
  virtual char *getTypeName();
  virtual char think();
	ControlWussie (char *name) ;
	virtual void set_target(int i) ;
  virtual void select_ship(Ship* ship_pointer, char* ship_name);
	virtual double check_danger();
	double weapon_range;
	double weapon_velocity;
	int special_type;
	double special_freq;
	};

class ControlSpyAI : public Control {	
	AI_Player *ai;
  public:
  virtual char *getTypeName();
  virtual char think();
  virtual void select_ship(Ship* ship_pointer, char* ship_name);
	ControlSpyAI (char *name) ;
  virtual void deinitialize();
	};

class ControlJoy : public Control {
	public:
  virtual void initialize();
  virtual void deinitialize();
	virtual char *getTypeName();
	virtual char think();
	int which_joystick;
	ControlJoy (char *name) ;
	};

//class ControlNet : public Control {
//	};

extern const char num_controls;
extern char **control_name;
Control *getController(int type, char *name);
int control2number(char *name) ;


#endif