#include <string.h>

#include "../melee.h"
#include "../libs.h"

#include "mnet.h"

#if defined NETWORK_BSD_TCP

#include <unistd.h>
#include <dos.h>
#include <lsck/lsck.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <arpa/nameser.h>
#include <fcntl.h>
#include <errno.h>

#define SOCKET int
#define INVALID_SOCKET -1
#define get_error() errno
#define set_error(a) (errno = a)
#define close_socket(a) close(a)
#define SOCKET_ERROR -1

#elif defined NETWORK_WINSOCK_TCP

#include "winsock.h"
#define get_error() WSAGetLastError()
#define set_error(a) WSASetLastError(a)
#define close_socket(a) closesocket(a)

#endif

#ifndef NETWORK_NONE

static SOCKET gsock = INVALID_SOCKET;
static sockaddr_in addr;
static int net_side = 0;

class text_window {
	public:
	BITMAP *bmp;
	FONT *f;
	int fore, back;
	int width, height;
	void print (char *s) {
		int l = text_length(f, s);
		int h = text_height(f);
		if (h == 0) return;
		BITMAP *tmp = create_bitmap(l, h);
		clear_to_color(tmp, back);
		textprintf (tmp, f, 0, 0, fore, "%s", s);
		for (int x = 0; x < l; x += bmp->w) {
			blit (bmp, bmp, 0, h, 0, 0, width, height - h);
			rectfill (bmp, 0, height - h, width - 1, height - 1, back);
			blit (tmp, bmp, x, 0, 0, height - h, l, h);
			net_side += 1;
			}
		destroy_bitmap(tmp);
		return;
		}
	void print (char *s, int d) {
		char buffy[2048] = "";
		sprintf (buffy, "%s %d", s, d);
		print(buffy);
		return;
		}
	text_window (BITMAP *src, int x, int y, int w, int h, 
			FONT *font_, int foreground, int background) {
		bmp = create_sub_bitmap(src, x, y, w, h);
		width = w;
		height = h;
		f = font_;
		fore = palette_color[foreground];
		back = palette_color[background];
		return;
		}
	~text_window() {
		destroy_bitmap(bmp);
		return;
		}
	};

static text_window *net_win = NULL;

void net_denagle () {
	BOOL t = true;
//	setsockopt ( gsock, SOL_SOCKET, TCP_NODELAY, (const char *)&t, sizeof(t));
	}

void net_error (char *s) {
	char buffy[256];
	int d = get_error();
	buffy[0] = 0;
	sprintf (buffy, "%d: %s\n", d, s);
	tw_error(buffy);
	}

int net_send (void *data, int size) {
	int i;
	i = send(gsock, (const char*)data, size, 0);
	return i == size;
	}

int net_recv (void *data, int size) {
	int i;
	i = recv(gsock, (char *)data, size, 0);
	return i;
	}

int net_send (char in) {
	int i;
	unsigned char data[4];
	data[0] = in;
	data[1] = 0x87;
	i = net_send(&data, sizeof(char) * 4);
	return i;
	}

int net_recv () {
	int i;
	unsigned char data[4] = {0,0,0,0};
	while (data[1] != 0x87) {
		i = net_recv(&data, sizeof(char) * 4);
		}
	return data[0];
	}

int net_pending(int i) {
	SOCKET s = gsock;
	fd_set read_set, write_set, except_set;
	timeval tmv;
	FD_ZERO(&read_set);
	FD_ZERO(&write_set);
	FD_ZERO(&except_set);
	if (i & 1) FD_SET ( s, &read_set);
	if (i & 2) FD_SET ( s, &write_set);
	if (i & 4) FD_SET ( s, &except_set);
	tmv.tv_sec = 0;
	tmv.tv_usec = 0;
	int r = select (0, &read_set, &write_set, &except_set, &tmv);
	return r;
	}

int net_listen () {
	int old_error = 0;
	net_side = 1;
	net_win->print ("listening... (press Escape to cancel)");
	rest(700);
	if (listen (gsock, 5) == -1)
		net_error ("listen failed\n");
	int tmp = sizeof(sockaddr_in);
	int i;
	while (true) {
		i = net_pending();
		if (i) break;
		if (key[KEY_ESC]) tw_error("escape pressed\n");
		}
	if ((i = accept (gsock, (sockaddr *) &addr, &tmp)) == INVALID_SOCKET) {
		tw_error("net_listen: accepting incoming connection failed\n");
		}
	close (gsock);
	gsock = i;
	net_denagle();
	return 0;
	}

int net_connect (char *target, int port) {
	int old_error = 0;
	net_side = 2;
  addr.sin_addr.s_addr = inet_addr(target);
  addr.sin_port = htons(port);
	net_win->print ("connecting to port ", port);
	net_win->print ("(press Escape to cancel)");
	net_win->print ("(and if that don't work, try Cntrl-Alt-Escape)");
	rest(700);
  while (connect (gsock, (sockaddr *) &addr, sizeof(sockaddr)) == SOCKET_ERROR) {
		if (key[KEY_ESC]) tw_error ("escape pressed\n");
		}
	net_denagle();
	return 0;
	}

int net_start (int port) {
	if (set_gfx_mode (GFX_GDI, 640, 480, 0, 0)) tw_error ("failed to init graphics\n");
	text_mode (-1);
	net_win = new text_window(screen, 64, 64, 640-128, 480-128, font, 15, 1);
	clear_to_color (net_win->bmp, palette_color[8]);
	WSADATA info;
	if (WSAStartup (MAKEWORD(1,1), &info) != 0) net_error("network startup failed");
	net_win->print ("network started");
	rest(700);
	gsock = socket( AF_INET, SOCK_STREAM, 0);
	if (gsock == INVALID_SOCKET) {
		tw_error ("socket allocation failed\n");
		}
	net_win->print ("socket allocated");
	rest(700);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = 0;
	memset(&(addr.sin_zero), 0, 8);
	if (port) {
		if (bind (gsock, (sockaddr *) &addr, sizeof(sockaddr) ) == INVALID_SOCKET) {
			tw_error ("bind to port failed\n");
			}
		else net_win->print ("socket bound to port", port);
		}
	else net_win->print ("no port binding requested");
	rest (700);
	return 0;
	}

int net_end() {
	if (gsock != INVALID_SOCKET) {
		net_send(127);
		close_socket(gsock);
		}
	gsock = INVALID_SOCKET;
	return 0;
	}
/*
int net_chat () {
	char sent[16] = "sent:       ";
	char recv[16] = "recieved:   ";
	net_win->print ("chatting");
	clear_keybuf();
	while (!key[KEY_ESC]) {
		while (!keypressed()) {
			if (net_pending()) {
				recv[10] = net_recv();
				net_win->print(recv);
				}
			}
		sent[10] = readkey() & 255;
		net_send(sent[10]);
		net_win->print (sent);
		}
	tw_error("networking not implemented yet\n");
	return 0;
	}
*/

void draw_rf(BITMAP *rfh, FONT *f, char *s) {
	int l = text_length(f, s);
	int h = text_height(f);
	rectfill (rfh, 0, 0, rfh->w, rfh->h, palette_color[4]);
	if (l < rfh->w) textprintf (rfh, f, 0, (rfh->h-h)/2, palette_color[15], "%s", s);
	else textprintf (rfh, f, rfh->w-l, (rfh->h-h)/2, palette_color[15], "%s", s);
	return;
	}

int net_chat () {
	int red_field_height = 10;
	int rfh = red_field_height;
	int exiting = false;
	char lbuffer[256] = "> ";
	char rbuffer[256] = "connection confirmed ; chat begun ; press Escape to exit";
	text_mode(-1);
	clear_to_color(net_win->bmp, 1);
	text_window *lwin = new text_window ( net_win->bmp, 0,                       0, 
			net_win->bmp->w, net_win->bmp->h/2 - rfh/2, font, 15, 1);
	text_window *rwin = new text_window( net_win->bmp, 0, net_win->bmp->h/2+rfh/2, 
			net_win->bmp->w, net_win->bmp->h/2 - rfh/2, font, 15, 1);
	BITMAP *rf = create_sub_bitmap(net_win->bmp, 0, net_win->bmp->h/2-rfh/2, 
			net_win->bmp->w, rfh);
	int x = 2;
	net_send(rbuffer, strlen(rbuffer));
	draw_rf(rf, font, lbuffer);
  while (!exiting) {
    if (keypressed()) {
      int ch = readkey();
      switch (ch & 255) {
        case 27:
					lbuffer[0] = 27;
					net_send ((void *)&lbuffer, 1);
					exiting = true;
					break;
        case 13:
          lbuffer[x] = 0;
					lwin->print(lbuffer);
					net_send(lbuffer, strlen(lbuffer));
					x = 2;
					lbuffer[x] = 0;
					draw_rf(rf, font, lbuffer);
          break;
        case 8:
          if (x == 2) break;
          x -= 1;
          lbuffer[x] = 0;
					if (!keypressed()) draw_rf(rf, font, lbuffer);
          break;
        default:
					if (x == 254) break;
          lbuffer[x] = ch & 255;
          x += 1;
					lbuffer[x] = 0;
					draw_rf(rf, font, lbuffer);
          break;
        }
      }
    if (net_pending()) {
			int i = net_recv(rbuffer, 255);
			if (i > 0) {
				rbuffer[i] = 0;
				rwin->print (rbuffer);
				if (rbuffer[0] == 27) exiting = true;
				}
			else tw_error ("net_chat: error\n");
			}
		}
	destroy_bitmap(rf);	
	delete rwin, lwin;
	delete net_win;
	return 0;
	}

#else

int net_chat () {
	tw_error("network not supported\n"); return 0;}
int net_listen () {
	tw_error("network not supported\n"); return 0;}
int net_connect (char *target, int port) {
	tw_error("network not supported\n"); return 0;}
int net_start (int port) {
	tw_error("network not supported\n"); return 0;}
int net_send (char data) {
	tw_error("network not supported\n"); return 0;}
int net_recv () {
	tw_error("network not supported\n"); return 0;}
int net_end () {
	return 0;}

#endif
