
#include "../melee.h"
#include "../id.h"
#include "../frame.h"

#include "mcbodies.h"
#include "mmain.h"
#include "mcontrol.h"

int hot_color[HOT_COLORS] =
{
  122, 123, 124, 125, 126, 127,
  42,  43,  44,  45,  46,  47
};

SpaceSprite *hotspotSprite;
SpaceSprite *kaboomSprite;

JGMOD *meleeVictory;

Ship::Ship(double ox, double oy, double shipAngle, int team,
  int shipRadarColor, ShipData *shipData, Control *cntrl) :
  SpaceObject(ox, oy, shipAngle, team, shipData->spriteShip),
  data(shipData),
  target(NULL),
  update_panel(FALSE), 
	target_pressed(FALSE), 
	control(cntrl)
{
  layer = LAYER_SHIPS;
  BITMAP *bmp[HOT_COLORS];
  int    i;
	nextkeys = 0xff;

  id = SPACE_SHIP;

	#ifdef SCP_DEBUG
  fprintf(debug_file, "* Ship::Ship -> Successful basic construction\n");
	fflush(debug_file);
	#endif
//  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  crew     = get_config_int("Ship", "Crew", 0);
  crew_max = get_config_int("Ship", "CrewMax", 0);
  batt     = get_config_int("Ship", "Batt", 0);
  batt_max = get_config_int("Ship", "BattMax", 0);

  recharge_amount  = get_config_int("Ship", "RechargeAmount", 0);
  recharge_rate    = get_config_int("Ship", "RechargeRate", 0);
  recharge_step    = 0;
  weapon_drain     = get_config_int("Ship", "WeaponDrain", 0);
  weapon_rate      = get_config_int("Ship", "WeaponRate", 0);
  weapon_sample    = 0;
  weapon_recharge  = 0;
  weapon_low       = FALSE;
  special_drain    = get_config_int("Ship", "SpecialDrain", 0);
  special_rate     = get_config_int("Ship", "SpecialRate", 0);
  special_sample   = 0;
  special_recharge = 0;
  special_low      = FALSE;

  hotspot_rate  = get_config_int("Ship", "HotspotRate", 0);
  hotspot_frame = 0;
  turn_rate     = (double)(get_config_int("Ship", "TurnRate", 0));
  turn_step     = 0.0;
  speed_max     = (double)(get_config_int("Ship", "SpeedMax", 0));
  accel_rate    = (double)(get_config_int("Ship", "AccelRate", 0));
  radar_color   = shipRadarColor;
  mass          = (double)(get_config_int("Ship", "Mass", 0));

  calculate_speed_max();
  calculate_accel_rate();
  calculate_turn_rate();
  scale_frame_rate(&weapon_rate);
  scale_frame_rate(&special_rate);
  scale_frame_rate(&recharge_rate);
  scale_frame_rate(&hotspot_rate);

  thrust       = FALSE;
  turn_left    = FALSE;
  turn_right   = FALSE;
  fire_weapon  = FALSE;
  fire_special = FALSE;

  panelBitmap[0] = create_bitmap(64, 100);
  blit(data->bitmapPanel[0], panelBitmap[0], 0, 0, 0, 0, 64, 100);

  for(i = 1; i < 7; i++) {
    panelBitmap[i] = create_bitmap(CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
    blit(data->bitmapPanel[i], panelBitmap[i], 0, 0, 0, 0,
      CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
  }

  angle = shipAngle;
  sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

  phase_count = 0;
  phase_step  = 0;

  for(i = 0; i < HOT_COLORS; i++) {
    bmp[i] = create_bitmap((int)(w), (int)(h));
    clear_to_color(bmp[i], bitmap_mask_color(bmp[i]));
    sprite->draw_character(0, 0, sprite_index, pallete_color[hot_color[i]], bmp[i]);
  }
  phaseSprite = new SpaceSprite(bmp, HOT_COLORS);

  for(i = 0; i < HOT_COLORS; i++)
    destroy_bitmap(bmp[i]);
	#ifdef SCP_DEBUG
  fprintf(debug_file, "* Ship::Ship -> Constructor exit\n");
	fflush(debug_file);
	#endif
}

Ship::~Ship()
{
  int i;

	#ifdef SCP_DEBUG
  fprintf(debug_file, "* Ship::~Ship -> Destructor begins\n");
	fflush(debug_file);
	#endif
  for(i = 0; i < 7; i++)
    destroy_bitmap(panelBitmap[i]);

  delete phaseSprite;
	#ifdef SCP_DEBUG
  fprintf(debug_file, "* Ship::~Ship -> Successful destruction\n");
	fflush(debug_file);
	#endif
}

 int Ship::getCrew()
{
  return(crew);
}

 int Ship::getBatt()
{
  return(batt);
}

 int Ship::crewPanelColor()
{
  return pallete_color[2];
}

 int Ship::battPanelColor()
{
  return pallete_color[4];
}

void Ship::locate()
{
  double r;

  do {
    x = (double)(random() % (int)(X_MAX));
    y = (double)(random() % (int)(Y_MAX));
    x = normal_x();
    y = normal_y();
    r = sqrt((x - spacePlanet->normal_x()) * (x - spacePlanet->normal_x()) +
             (y - spacePlanet->normal_y()) * (y - spacePlanet->normal_y()));
  } while(r < GRAVITY_RANGE);
}

void Ship::calculate()
{
  if(phase_count < PHASE_MAX) {
    calculate_phase();
    return;
  }

  if(batt < batt_max) {
    recharge_step++;
    if(recharge_step == recharge_rate) {
      batt += recharge_amount;
      recharge_step = 0;
    }
  }

  if(weapon_recharge > 0)
    weapon_recharge--;
  if(special_recharge > 0)
    special_recharge--;

	if (!target_pressed) {
		if (target_next) {
			if (control->num_targets)
				control->set_target((control->active_target + 1) % control->num_targets);
			}
		else if (target_prev) {
			if (control->num_targets)
				control->set_target((control->active_target + control->num_targets - 1) % control->num_targets);
			}
		else if (target_close) {
			if (control->num_targets) {
				int i, j;
				double r = 99999;
				double d;
				for (i = 0; i < control->num_targets; i += 1) {
					d = distance(control->target[i]);
					if (d < r) {
						r = d;
						j = i;
						}
					}
				control->set_target(j);
				}
			}
		}
	target_pressed = target_next | target_prev | target_close;

  calculate_thrust();
  calculate_turn_left();
  calculate_turn_right();
  calculate_fire_weapon();
  calculate_fire_special();

  while(fabs(turn_step) > 5.625) {
    if(turn_step < 0.0) {
      angle -= 5.625;
      turn_step += 5.625;
    } else
    if(turn_step > 0.0) {
      angle += 5.625;
      turn_step -= 5.625;
    }

    if(angle < 0.0)
      angle += 360.0;
    if(angle >= 360.0)
      angle -= 360.0;
  }

  sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

  calculate_gravity();
  calculate_hotspots();

  SpaceObject::calculate();
}

void Ship::animate(Frame *space, Frame *radar)
{
  if(phase_count < PHASE_MAX)
    return;
  SpaceObject::animate(space, radar);
	if (panel[hero] && (this == panel[hero]->ship) && target) {
		double x, y, r;
		r = 160 * space_zoom;
		x = target->normal_x() - space_x;
		x = normalize(x, X_MAX) * space_zoom;
		if ((x >= -r) && (x < VIEW_X + r)) {
			y = target->normal_y() - space_y;
			y = normalize(y, Y_MAX) * space_zoom;
			if ((y >= -r) && (y < VIEW_Y + r)) {
				vline (space->frame, x, y-r, y+r, pallete_color[4]);
			  space->add_line((int)(x), (int)(y-r), (int)(x), (int)(y+r));
				hline (space->frame, x-r, y, x+r, pallete_color[4]);
			  space->add_line((int)(x-r), (int)(y), (int)(x+r), (int)(y));
				}
			}
		}
	}

void Ship::handle_damage()
{
  batt -= fuel_sap;
  fuel_sap = 0;
  if(batt < 0)
    batt = 0;
  if(speed_loss > 0.0) {
    speed_max *= (1.0 - (1.0 / (double)(speed_loss)));
    turn_rate /= (1.0 + (1.0 / (double)(speed_loss)));
    hotspot_rate++;
    speed_loss = 0.0;
  }
  crew -= damage;
  damage = 0;
  crew -= directDamage;
  directDamage = 0;
  crew += repair;
  repair = 0;
  if(crew > crew_max)
    crew = crew_max;
  if(crew <= 0) {
    crew  = 0;
    state = 0;
    tw_sample((SAMPLE *)(melee[MELEE_BOOMSHIP].dat));
    spaceFrame->addSpaceObject(new Animation(x, y,
      kaboomSprite, KABOOM_FRAMES, 5, LAYER_EXPLOSIONS));
		if(target != NULL)
      meleeVictory = target->data->moduleVictory;
    else
      meleeVictory = NULL;
		#ifdef SCP_DEBUG
		fprintf(debug_file, "x Ship -> Death sequence begins\n");
		fflush(debug_file);
		#endif
  }
}

 void Ship::calculate_turn_rate()
{
  turn_rate = 20.0 / ((turn_rate + 1.0) * 5.0);
}

 void Ship::calculate_speed_max()
{
  speed_max = speed_max / 7.5;
}

 void Ship::calculate_accel_rate()
{
  accel_rate = accel_rate / 100.0;
}

 void Ship::materialize() {}

 void Ship::calculate_phase()
{
  if(phase_count < PHASE_MAX) {
    if(phase_step > 0)
      phase_step--;
    else {
      phase_step = PHASE_DELAY;
      spaceFrame->addSpaceObject(new Animation(
        x - (cos(angle * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
        y - (sin(angle * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
        phaseSprite, HOT_COLORS, 5, LAYER_SHIPS));
      phase_count++;
      if(phase_count == PHASE_MAX)
        materialize();
    }
  }
};

 void Ship::calculate_thrust()
{
  double v;

  if(thrust) {
    vx += (cos(angle * ANGLE_RATIO) * accel_rate);
    vy += (sin(angle * ANGLE_RATIO) * accel_rate);

    v = sqrt((vx * vx) + (vy * vy));

    if((v > speed_max) && (v != 0.0)) {
      vx = vx * speed_max / v;
      vy = vy * speed_max / v;
    }

    driftvx *= 0.99;
    driftvy *= 0.99;
  }
}

 void Ship::calculate_turn_left()
{
  if(turn_left)
    turn_step -= turn_rate;
}

 void Ship::calculate_turn_right()
{
  if(turn_right)
    turn_step += turn_rate;
}

void Ship::calculate_fire_weapon()
{
  weapon_low = FALSE;

  if(fire_weapon) {
    if(batt < weapon_drain) {
      weapon_low = TRUE;
      return;
    }

    if(weapon_recharge != 0)
      return;

    if(!activate_weapon())
      return;

    batt -= weapon_drain;
    weapon_recharge = weapon_rate;

    stop_sample(data->sampleWeapon[weapon_sample]);
		tw_sample(data->sampleWeapon[weapon_sample]);
  }
}

void Ship::calculate_fire_special()
{
  special_low = FALSE;

  if(fire_special) {
    if(batt < special_drain) {
      special_low = TRUE;
      return;
    }

    if(special_recharge != 0)
      return;

    if(!activate_special())
      return;

    batt -= special_drain;
    special_recharge = special_rate;

    stop_sample(data->sampleSpecial[special_sample]);
    tw_sample(data->sampleSpecial[special_sample]);
  }
}

void Ship::calculate_gravity()
{
  double r;
  double alpha;

  r = sqrt((normal_x() - spacePlanet->normal_x()) *
           (normal_x() - spacePlanet->normal_x()) +
           (normal_y() - spacePlanet->normal_y()) *
           (normal_y() - spacePlanet->normal_y()));

  if(r < GRAVITY_RANGE) {
    // calculate gravity angle
    alpha = atan((normal_y() - spacePlanet->normal_y()) /
      (normal_x() - spacePlanet->normal_x()));
    if(normal_x() > spacePlanet->normal_x())
      alpha += PI;

    // apply gravity
    driftvx += (cos(alpha) * (GRAVITY_FORCE / (r * 250)));
    driftvy += (sin(alpha) * (GRAVITY_FORCE / (r * 250)));
  }
}

void Ship::calculate_hotspots()
{
  if((thrust) && (hotspot_frame == 0)) {
    spaceFrame->addSpaceObject(new Animation(
      normal_x() - (cos(angle * ANGLE_RATIO) * w / 2.5),
      normal_y() - (sin(angle * ANGLE_RATIO) * h / 2.5),
      hotspotSprite, HOTSPOT_FRAMES, 5, LAYER_HOTSPOTS));
    hotspot_frame = hotspot_rate;
  }

  if(hotspot_frame > 0)
    hotspot_frame--;
}

 int Ship::activate_weapon()
{
  return(TRUE);
}

inline int Ship::activate_special()
{
  return(TRUE);

}
