
#include <allegro.h>


#include "../melee.h"
#include "../libs.h"
#include "../ppcol.h"
#include "../frame.h"

#include "mmain.h"

PPCOL_MASK *pix;

SpaceSprite::SpaceSprite(DATAFILE *sprites, int sprite_count)
{
  int i;

  count = sprite_count;

  m = new PPCOL_MASK*[count];
	//MYCODE
  b = new BITMAP*[count];

  w = ((RLE_SPRITE *)sprites[0].dat)->w;
  h = ((RLE_SPRITE *)sprites[0].dat)->h;

  for(i = 0; i < count; i++) {

    b[i] = create_bitmap(w, h);

    clear_to_color(b[i], bitmap_mask_color(b[i]));

    tw_draw_rle_sprite(b[i], (RLE_SPRITE *)(sprites[i].dat), 0, 0);


    m[i]                 = create_ppcol_mask(b[i]);

  }

	
	//MYCOMMENT
  //destroy_bitmap(bmp1);
}

SpaceSprite::SpaceSprite(BITMAP **bmp, int bmp_count)
{
  int i;

  count = bmp_count;

  m = new PPCOL_MASK*[count];
	//MYCODE
  b = new BITMAP*[count];

  w = bmp[0]->w;
  h = bmp[0]->h;


  for(i = 0; i < count; i++) {

    b[i] = create_bitmap(w, h);

    clear_to_color(b[i], bitmap_mask_color(b[i]));

    tw_draw_sprite(b[i], bmp[i], 0, 0);

    m[i]                 = create_ppcol_mask(b[i]);

  }

}

SpaceSprite::~SpaceSprite()
{
  int i;

  #ifdef SCP_DEBUG
  fprintf(debug_file, "? SpaceSprite -> Destructor\n");
  fflush(debug_file);
  #endif
  
  for(i = 0; i < count; i++) {
    destroy_ppcol_mask(m[i]);
  }

  #ifdef SCP_DEBUG
  fprintf(debug_file, "? SpaceSprite -> Destroyed successfully\n");
  fflush(debug_file);
  #endif
  //MYCODE
  for(i = 0; i < count; i++) {
    destroy_bitmap(b[i]);
  }
}

BITMAP *SpaceSprite::get_bitmap(int index)
{
  return(b[index]);
}

void SpaceSprite::animate(int x, int y, int index, Frame *space)
{
  int spr_w = int(double(w) * space_zoom);
  int spr_h = int(double(h) * space_zoom);
  
  if(spr_w < 1) spr_w = 1;
  if(spr_h < 1) spr_h = 1;


  if (antia_on) {
    aa_stretch_sprite(space->frame, b[index], x, y, spr_w, spr_h);
  } else {
    stretch_sprite(space->frame, b[index], x, y, spr_w, spr_h);
  }
  
  space->add_box(x, y, spr_w, spr_h);
}

void SpaceSprite::draw_character(int x, int y, int index, int color,
  BITMAP *bmp)
{
  PPCOL_MASK *mask = m[index];

#ifdef SLOW_SPRITE
  int i,j;
  for(i = 0; i < mask->h; i++)
    for(j = 0; j < mask->w; j++)
      if( check_ppmask_collision(mask, pix, 0, 0, i, j) )
        putpixel(bmp, x + i, y + j, color);
#else
  int row, chunk;
  for(row = 0; row < mask->h; row++) {
    for(chunk = 0; chunk <= mask->max_chunk; chunk++) {
      int c = mask->sp_mask[row][chunk];
    int cx = x + (chunk << 5);
      while (c) {
        if (c & 0x80000000) putpixel(bmp, cx, y + row, color);
        c = c << 1;
        cx += 1;
        }
      }
    }
#endif
}

//MYCODE
 void SpaceSprite::draw_character_stretch(int x, int y, int index, int color,double zoom,
  BITMAP *bmp)
{
  PPCOL_MASK *mask = m[index];

#ifdef SLOW_SPRITE
  int i, j; 
  for(i = 0; i < mask->h; i++)
    for(j = 0; j < mask->w; j++)
      if( check_ppmask_collision(mask, pix, 0, 0, i, j) ) 
        putpixel(bmp, x + int(i * zoom), y + int(j * zoom), color);
#else
  int zoom_factor;
  const int ebits = 8;
  int         row, chunk, cx, ry;
  zoom_factor = int(zoom * (1 << (ebits)));
  for(row = 0; row < mask->h; row++) {
    cx = x << ebits;
  ry = y + ((row * zoom_factor) >> ebits);
    for(chunk = 0; chunk <= mask->max_chunk; chunk++) {
      int c = mask->sp_mask[row][chunk];
      cx = (x << ebits) + chunk * (zoom_factor << 5);
	  while (c) {
        if (c & 0x80000000) putpixel(bmp, cx >> ebits, ry, color);
        c = c << 1;
		cx += zoom_factor;
        }
      }
    }
#endif	
}

//MYCODE
void SpaceSprite::animate_character(int x, int y, int index, int color,
  Frame *space)
{
  int spr_w = int( double(w) * space_zoom)+2;
  int spr_h = int( double(h) * space_zoom)+2;

  if(spr_w < 1) spr_w = 1;
  if(spr_h < 1) spr_h = 1;

  #ifdef SCP_DEBUG
  fprintf(debug_file, "? SpaceSprite -> animate_character\n");
  fflush(debug_file);
  #endif

  draw_character_stretch(x,y,index,color,space_zoom,space->frame);

  space->add_box(x, y, spr_w, spr_h);
}

 int SpaceSprite::collide(int x, int y, int i, int ox, int oy, int oi,
  SpaceSprite *other)
{
  return(check_ppmask_collision(m[i], other->m[oi], x, y, ox, oy));
}

int line_collide;
int rect_x;
int rect_y;
int rect_w;
int rect_h;
int collide_x;
int collide_y;

PPCOL_MASK *rect_mask;

void check_line_collision(BITMAP *bmp, int x, int y, int d)
{
  int row, col;

  if(line_collide)
    return;

  col = x - rect_x;
  row = y - rect_y;

#ifdef SLOW_SPRITE
  if((col >= 0) && (col < rect_w) && (row >= 0) && (row < rect_h)) 
    if(check_ppmask_collision(rect_mask, pix, 0, 0, col, row)) {
      line_collide = TRUE;
      collide_x = x;
      collide_y = y;
    }
#else
  if((col >= 0) && (col < rect_w) &&
     (row >= 0) && (row < rect_h)) {
    if((rect_mask->sp_mask[row][col >> 5] & (0x80000000 >> (col & 31))) != 0) {
      line_collide = TRUE;
      collide_x = x;
      collide_y = y;
    }
  }
#endif
}

int SpaceSprite::collide_line(int lx1, int ly1, int *lx2, int *ly2,
  int sx, int sy, int sindex)
{
  line_collide = FALSE;
  rect_x = sx - (w / 2);
  rect_y = sy - (h / 2);
  rect_w = w;
  rect_h = h;
  rect_mask = m[sindex];
  do_line(NULL, lx1, ly1, *lx2, *ly2, 0, check_line_collision);
  if(line_collide) {
    *lx2 = collide_x;
    *ly2 = collide_y;
    return(TRUE);
  }

  return(FALSE);
}

 int SpaceSprite::frames()
{
  return(count);
}

 double SpaceSprite::width()
{
  return((double)(w));
}

 double SpaceSprite::height()
{
  return((double)(h));
}

SpaceRotatingSprite::SpaceRotatingSprite(DATAFILE *sprites, int sprite_count) :
  SpaceSprite(*this)
{
  int i, j;
  BITMAP *bmp, *bmp1;

  count = sprite_count;

  m = new PPCOL_MASK*[count * 64];
  b = new BITMAP*    [count * 64];

  w = ((RLE_SPRITE *)sprites[0].dat)->w;
  h = ((RLE_SPRITE *)sprites[0].dat)->h;

  bmp  = create_bitmap(w, h);

  for(i = 0; i < count; i++) {
    clear_to_color(bmp, bitmap_mask_color(bmp));
    tw_draw_rle_sprite(bmp, (RLE_SPRITE *)(sprites[i].dat), 0, 0);
    for(j = 0; j < 64; j++) {

      bmp1 = create_bitmap(w, h);
			clear_to_color(bmp1, bitmap_mask_color(bmp1));

      rotate_sprite(bmp1, bmp, 0, 0, itofix(4 * j));

      m[(i * 64) + j]                 = create_ppcol_mask(bmp1);
      b[(i * 64) + j]=bmp1;
    }
  }

  count *= 64;

  destroy_bitmap(bmp);
}
