

#include "ship.h"

class ChmmrAvatar : public Ship {
  double weaponRange;
  int    weaponFrames;
  int    weaponDamage;

  double       specialForce;
  SpaceSprite *specialSprite;

  double extraRange;
  int    extraDamage;
  int    extraFrames;
  int    extraRechargeRate;
  int    extraColor;
  int    extraArmour;

  public:
  ChmmrAvatar(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void materialize();

  virtual void calculate();
};

class ChmmrLaser : public Laser {
  public:
  ChmmrLaser(double langle, double lrange, int ldamage, int lfcount,
    SpaceLocation *opos, double rel_x, double rel_y);
};

class ChmmrBeam : public SpaceObject {
  int frame;
  int frame_count;
  Ship *ship;
  Ship *target;

  public:
  ChmmrBeam(Ship *oship, int oframes);

  virtual void calculate();
  virtual void animate(Frame *space, Frame *radar);
};

class ChmmrZapSat : public SpaceObject {
  int frame;
  Ship *ship;

  double lRange;
  int    lDamage;
  int    lFrames;
  int    lRechargeRate;
  int    lRecharge;
  int    lColor;
  int    armour;

  public:
  ChmmrZapSat(double oangle, double orange, int odamage, int oframes,
    int orechargerate, int ocolor, int oarmour, Ship *oship,
    SpaceSprite *osprite);

  virtual void calculate();
  virtual void handle_damage();
};

ChmmrAvatar::ChmmrAvatar(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl) {
  
  weaponRange  = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponFrames = get_config_int("Weapon", "Frames", 0);
  weaponDamage = get_config_int("Weapon", "Damage", 0);

  specialForce = (1.0 / get_config_float("Special", "Force", 1)) * 7 / VELOCITY_RATIO;

  extraRange        = get_config_float("Extra", "Range", 0) * RANGE_RATIO;
  extraFrames       = get_config_int("Extra", "Frames", 0);
  extraDamage       = get_config_int("Extra", "Damage", 0);
  extraRechargeRate = get_config_int("Extra", "RechargeRate", 0);
  extraColor        = get_config_int("Extra", "Color", 0);
  extraArmour       = get_config_int("Extra", "Armour", 0);
  scale_frame_rate(&extraRechargeRate);
}

int ChmmrAvatar::activate_weapon()
{
  spaceFrame->addSpaceLine(new ChmmrLaser(angle, weaponRange,
    weaponDamage, weaponFrames, this, 0.0, -25.0));
  return(TRUE);
}

int ChmmrAvatar::activate_special()
{
  spaceFrame->addSpaceObject(new ChmmrBeam(this, 5));
  return(TRUE);
}

void ChmmrAvatar::materialize()
{
  spaceFrame->addSpaceObject(new ChmmrZapSat(0.0, extraRange,
    extraDamage, extraFrames, extraRechargeRate, extraColor, extraArmour, this,
    data->spriteExtra));
  spaceFrame->addSpaceObject(new ChmmrZapSat(120.0, extraRange,
    extraDamage, extraFrames, extraRechargeRate, extraColor, extraArmour, this,
    data->spriteExtra));
  spaceFrame->addSpaceObject(new ChmmrZapSat(240.0, extraRange,
    extraDamage, extraFrames, extraRechargeRate, extraColor, extraArmour, this,
    data->spriteExtra));
}

void ChmmrAvatar::calculate()
{
  if((special_recharge != 0) && (target != NULL) && (!target->isAlly(this)))
    target->pull(this, specialForce);
  Ship::calculate();
}

ChmmrLaser::ChmmrLaser(double langle, double lrange, int ldamage,
  int lfcount, SpaceLocation *opos, double rel_x, double rel_y) :
  Laser(langle, pallete_color[hot_color[random() % HOT_COLORS]], lrange,
    ldamage, lfcount, opos, rel_x, rel_y)
{
}

ChmmrBeam::ChmmrBeam(Ship *oship, int oframes) :
  SpaceObject(oship->normal_x(), oship->normal_y(), oship->get_angle(), -1,
    sparkSprite),
  frame(0),
  frame_count(oframes),
  ship(oship),
  target(oship->target)
{
	collide_flag_anyone = 0;
  if(!ship->exists()) {
    state = 0;
    return;
  }

  target = ship->target;
  if((target == NULL) || (target->isAlly(this))) {
    state = 0;
    return;
  }
}

void ChmmrBeam::calculate()
{
  if(!ship->exists()) {
    state = 0;
    return;
  }

  target = ship->target;
  if((!target) || (target->isAlly(this))) {
    state = 0;
    return;
  }

  x = ship->normal_x();
  y = ship->normal_y();
  frame++;
  if(frame == frame_count)
    state = 0;
}

void ChmmrBeam::animate(Frame *space, Frame *radar)
{
  const int beam_color[5] = { 80, 81, 82, 83, 84 };
  int i;
  double length;

  length = sqrt((target->get_vx() * target->get_vx()) +
    (target->get_vy() * target->get_vy())) + (target->width() / 4.0);

  for(i = 0; i < 4; i++)
    target->data->spriteShip->animate_character(
      (int)(wcorner(target->normal_x() + ((double)(i + 1) *
        cos((trajectory_angle(target) - 180.0) * ANGLE_RATIO) * length),
        target->width())),
      (int)(hcorner(target->normal_y() + ((double)(i + 1) *
        sin((trajectory_angle(target) - 180.0) * ANGLE_RATIO) * length),
        target->height())),
      target->getSpriteIndex(), pallete_color[beam_color[i]], space);
}

ChmmrZapSat::ChmmrZapSat(double oangle, double orange, int odamage,
  int oframes, int orechargerate, int ocolor, int oarmour, Ship *oship,
  SpaceSprite *osprite) :
  SpaceObject(0.0, 0.0, 0.0, oship->allyFlag(), osprite),
  ship(oship),
  lRange(orange),
  lDamage(odamage),
  lFrames(oframes),
  lRechargeRate(orechargerate),
  lRecharge(0),
  lColor(ocolor),
  armour(oarmour)
{
	layer = LAYER_SHIPS;
	collide_flag_anyone = ~(bit(LAYER_SHIPS) | bit(LAYER_CBODIES));
	id |= CHMMR_SPEC;
  angle = oangle;
  x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * 100.0);
  y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * 100.0);

  if(!ship->exists())
    state = 0;
}

void ChmmrZapSat::calculate()
{
  int i;
  SpaceObject *o;

  if(lRecharge > 0)
    lRecharge--;

  SpaceObject::calculate();

  if(!ship->exists()) {
    state = 0;
    return;
  }

  x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * 100.0);
  y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * 100.0);
  angle += 1.0;
  if(angle == 360.0)
    angle = 0.0;
  sprite_index++;
  if(sprite_index == 64)
    sprite_index = 0;

  for(i = 0; i < SPACE_LAYERS; i +=1) {
    o = spaceFrame->getFirstObject(i);
    while((lRecharge == 0) && (o != NULL)) {
      if(( !o->isAlly(this)) && (distance(o) < lRange) && (o->layer != LAYER_CBODIES)) {
				SpaceLine *l =
          spaceFrame->addSpaceLine(new Laser(trajectory_angle(o),
					pallete_color[lColor], distance(o), 0, lFrames, this, 0.0, 0.0));
				if (o->canCollide(l)) {
					o->damage += lDamage;
					o->handle_damage();
					tw_sample((SAMPLE *)(melee[MELEE_BOOM + 0].dat));
					lRecharge = lRechargeRate;
					i = SPACE_LAYERS;
					}
				else l->state = 0;
				}
			o = o->next;
			}
		}
	}

void ChmmrZapSat::handle_damage() {
  if((damage > 0) || (directDamage > 0)) {
    armour -= damage;
    armour -= directDamage;
    damage = 0;
    directDamage = 0;
    if(armour <= 0) {
      armour = 0;
      state = 0;
      spaceFrame->addSpaceObject(new Animation(x, y,
        kaboomSprite, KABOOM_FRAMES, 5, LAYER_EXPLOSIONS));
      stop_sample(ship->data->sampleExtra[0]);
      tw_sample(ship->data->sampleExtra[0]);
    }
  }
}

Ship *getChmmrAvatar(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new ChmmrAvatar(sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO ChmmrAvatarInfo =
{
  "chmav",
  "Chmmr Avatar",
  getChmmrAvatar
};
