
#include "ship.h"

class DruugeMauler : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponDriftVelocity;

  public:
  DruugeMauler(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual int activate_weapon();
  virtual void calculate_fire_special();
};

class DruugeMissile : public Missile {
  public:
  DruugeMissile(double ox, double oy, double oangle, double ov, int odamage,
      double weaponDriftVelocity, double orange, int oarmour, Ship *oship,
      SpaceSprite *osprite);
  virtual void inflict_damage (SpaceObject *other);
  double kick;
};

DruugeMauler::DruugeMauler(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl) {

  weaponRange         = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity      = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage        = get_config_int("Weapon", "Damage", 0);
  weaponArmour        = get_config_int("Weapon", "Armour", 0);
  weaponDriftVelocity = get_config_float("Weapon", "DriftVelocity", 0) * 10 / VELOCITY_RATIO;
}

int DruugeMauler::activate_weapon() {
  spaceFrame->addSpaceObject(new DruugeMissile(
      0.0, -(height() / 2.0), angle, weaponVelocity, weaponDamage,
      weaponDriftVelocity, weaponRange, weaponArmour, this,
      data->spriteWeapon));
  vx -= (cos(angle * ANGLE_RATIO) * weaponDriftVelocity) / mass;
  vy -= (sin(angle * ANGLE_RATIO) * weaponDriftVelocity) / mass;

  return(TRUE);
}

void DruugeMauler::calculate_fire_special() {
  if((fire_special) && (crew > 1) && (batt < batt_max) &&
    (special_recharge == 0)) {

    batt += special_drain;
    if(batt > batt_max)
      batt = batt_max;
    crew--;

    special_recharge = special_rate;

    stop_sample(data->sampleSpecial[0]);
    tw_sample(data->sampleSpecial[0]);
  }
}

DruugeMissile::DruugeMissile(double ox, double oy, double oangle, double ov,
    int odamage, double weaponDriftVelocity, double orange, int oarmour,
    Ship *oship, SpaceSprite *osprite) :
    Missile(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship, osprite),
    kick(weaponDriftVelocity)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 5;
  explosionFrameSize  = 5;
}
void DruugeMissile::inflict_damage (SpaceObject *other) {
  if (other->getID() == SPACE_PLANET) other->pull(this, -kick * 9999);
  other->pull(angle, kick);
  Missile::inflict_damage(other);
}

Ship *getDruugeMauler(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new DruugeMauler(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO DruugeMaulerInfo = {
  "druma",
  "Druuge Mauler",
  getDruugeMauler
};
