
#include "ship.h"

class EarthlingCruiser : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponTurnRate;

  int    specialColor;
  double specialRange;
  int    specialFrames;
  int    specialDamage;

  public:
  EarthlingCruiser(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
};

class EarthlingMissile : public HomingMissile {
  public:
  EarthlingMissile(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);
};

EarthlingCruiser::EarthlingCruiser(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl) {

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponTurnRate = get_config_float("Weapon", "TurnRate", 0);

  specialColor  = get_config_int("Special", "Color", 0);
  specialRange  = get_config_float("Special", "Range", 0) * RANGE_RATIO;
  specialFrames = get_config_int("Special", "Frames", 0);
  specialDamage = get_config_int("Special", "Damage", 0);

	collide_flag_ally = bit(LAYER_SHOTS);
}

int EarthlingCruiser::activate_weapon() {
  spaceFrame->addSpaceObject(new EarthlingMissile(
    0.0, -(height() * 1.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, weaponTurnRate, this, data->spriteWeapon));
  return(TRUE);
}

int EarthlingCruiser::activate_special() {
  int fire = FALSE, i;
  SpaceObject *o;

  for(i = 0; i < SPACE_LAYERS; i++) {
    o = spaceFrame->getFirstObject(i);
    while(o != NULL) {
				if( (!o->isAlly(this)) && (distance(o) < specialRange) &&
					canCollide(o)) {
				SpaceLine *l = spaceFrame->addSpaceLine(new Laser(trajectory_angle(o),
						pallete_color[specialColor], distance(o), 0,
						specialFrames, this, 0.0, 10.0));
				if (o->canCollide(l)) {
					o->damage += specialDamage;
					o->handle_damage();
					fire = TRUE;
					} else l->state = 0;
				}
			o = o->next;
			}
		}
	if (fire) tw_sample((SAMPLE *)(melee[MELEE_BOOM + 0].dat));

	return(fire);
	}

EarthlingMissile::EarthlingMissile(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, double otrate,
  Ship *oship, SpaceSprite *osprite) :
  HomingMissile(ox, oy, oangle, ov, odamage, orange, oarmour, otrate, oship,
    osprite)
{
	collide_flag_ally = bit(LAYER_SHIPS) | bit(LAYER_SHOTS);
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 10;
  explosionFrameSize  = 5;
}

Ship *getEarthlingCruiser(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new EarthlingCruiser(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO EarthlingCruiserInfo =
{
  "earcr",
  "Earthling Cruiser",
  getEarthlingCruiser
};
