
#include "ship.h"

class KohrAhBlade;

class KohrAhMarauder : public Ship
{
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  int          weaponFired;
  KohrAhBlade *weaponObject;

  double       specialVelocity;
  int          specialDamage;
  int          specialArmour;
  double       specialRange;

  public:
  KohrAhMarauder(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();

  int numblades;
  int maxblades;
};

class KohrAhBlade : public AnimatedShot {
  int       passive;
  double    passiveRange;
  double    passiveVelocity;

  public:
  KohrAhBlade(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship,
    SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void animateExplosion();

  void disengage();
};

class KohrAhBladeDecay : public Animation {
  public:
  KohrAhBladeDecay(double ox, double oy, double ovx, double ovy,
    SpaceSprite *osprite, int ofcount, int ofsize);
};

class KohrAhFRIED : public Shot {
  int frame_count;

  public:
  KohrAhFRIED(double oangle, double ov, int odamage, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount);

  virtual void calculate();
	virtual void inflict_damage(SpaceObject *other);
};

void KohrAhFRIED::inflict_damage(SpaceObject *other) {
	if ((other->getID() & ~SPACE_MASK) == CHMMR_SPEC) other->state = 0;
	Shot::inflict_damage(other);
	return;
	}

KohrAhMarauder::KohrAhMarauder(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{
  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponObject   = NULL;
  weaponFired    = FALSE;

  specialRange    = get_config_float("Special", "Range", 0) * RANGE_RATIO;
  specialVelocity = get_config_float("Special", "Velocity", 0) / VELOCITY_RATIO;
  specialDamage   = get_config_int("Special", "Damage", 0);
  specialArmour   = get_config_int("Special", "Armour", 0);

  numblades = 0;
  maxblades = 16;
}

int KohrAhMarauder::activate_weapon()
{
  if(weaponFired) return(FALSE);

  if (numblades >= maxblades) {
    numblades = 0;
    int i;
    for(i = 0; i < SPACE_LAYERS; i++) {
      SpaceObject *o = spaceFrame->getFirstObject(i);
      while(o != NULL) {
        if (isAlly(o) && ((o->getID() & SPACE_MASK) == SPACE_SHOT) && o->exists())
          numblades += 1;
        o = o->next;
        }
      }
    int sofar = 0;
    for(i = 0; i < SPACE_LAYERS; i++) {
      SpaceObject *o = spaceFrame->getFirstObject(i);
      while((o != NULL) && (sofar < numblades)) {
        if (isAlly(o) && ((o->getID() & SPACE_MASK) == SPACE_SHOT)) {
          sofar += 1;
          if (sofar >= maxblades) o->state = 0;
          }
        o = o->next;
        }
      if (numblades > maxblades) numblades = maxblades;
      }
    }
  
  weaponObject = new KohrAhBlade(0.0, -(h / 2.0), angle, weaponVelocity,
      weaponDamage, weaponRange, weaponArmour, this, data->spriteWeapon,
      10, 4);
  spaceFrame->addSpaceObject(weaponObject);
  weaponFired = TRUE;
  numblades += 1;
  return(TRUE);
}

int KohrAhMarauder::activate_special()
{
  int i;

  for(i = 0; i < 16; i++)
    spaceFrame->addSpaceObject(new KohrAhFRIED(
      ((double)(i) * 22.5) - 180.0, specialVelocity, specialDamage,
      specialRange, specialArmour, this, data->spriteSpecial, 20));

  return(TRUE);
}

void KohrAhMarauder::calculate()
{
  Ship::calculate();
  if((weaponFired) && (!fire_weapon)) {
    weaponObject->disengage();
    weaponFired = FALSE;
    weaponObject = NULL;
  }
}

KohrAhBlade::KohrAhBlade(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(ox, oy, oangle, ov, odamage, -1.0, oarmour, oship, oship,
    osprite, ofcount, ofsize),
  passive(FALSE),
  passiveRange(orange)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 20;
  explosionFrameSize  = 4;
}

void KohrAhBlade::calculate()
{
  int locked = FALSE, i;
  SpaceObject *o, *t = NULL;
  double oldrange = 999999;

  AnimatedShot::calculate();

  if(passive) {
    for(i = 0; (i < SPACE_LAYERS) && (!locked); i++) {
      o = spaceFrame->getFirstObject(i);
      while(o != NULL) {
        if (!o->isAlly(this) && (distance(o) < oldrange) && (o->allyFlag() != -1)) {
          if (distance(o) < passiveRange) {
            t = o;
            oldrange = distance(o);
          }
        }
        o = o->next;
      }
    }

    if (t) {
      angle = trajectory_angle(t);
      vx = (v / 10.0) * (cos(angle * ANGLE_RATIO));
      vy = (v / 10.0) * (sin(angle * ANGLE_RATIO));
    } else {
      vx = 0;
      vy = 0;
    }
  }
  return;
}

void KohrAhBlade::animateExplosion()
{
  spaceFrame->addSpaceObject(new KohrAhBladeDecay(x, y,
    vx, vy, explosionSprite, explosionFrameCount, explosionFrameSize));
}

void KohrAhBlade::disengage()
{
  passive = TRUE;
}

KohrAhBladeDecay::KohrAhBladeDecay(double ox, double oy, double ovx,
  double ovy, SpaceSprite *osprite, int ofcount, int ofsize) :
  Animation(ox, oy, osprite, ofcount, ofsize, LAYER_EXPLOSIONS)
{
  vx = ovx;
  vy = ovy;
}

KohrAhFRIED::KohrAhFRIED(double oangle, double ov, int odamage,
  double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount) :
  Shot(0.0, 0.0, oangle, ov, odamage, orange, oarmour, oship, oship, osprite),
  frame_count(ofcount)
{
  x += cos(angle * ANGLE_RATIO) * 50.0;
  y += sin(angle * ANGLE_RATIO) * 50.0;
}

void KohrAhFRIED::calculate() {
  sprite_index = (int)((d / range) * (double)(frame_count - 1));
  Shot::calculate();
}

Ship *getKohrAhMarauder(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new KohrAhMarauder(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO KohrAhMarauderInfo =
{
  "kohma",
  "Kohr-Ah Marauder",
  getKohrAhMarauder
};
