
#include "ship.h"

#include "../melee/mmain.h"

#define MAX_MARINES 8

class OrzNemesis;
class OrzMissile;
class OrzMarine;

class OrzNemesis : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialAccelRate;
  double       specialSpeedMax;
  int          specialHotspotRate;
  int          specialArmour;
  OrzMarine   *marine[MAX_MARINES];

  double       turretAngle;

  public:
  OrzNemesis(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual void calculate();
  virtual void animate(Frame *space, Frame *radar);

  virtual void calculate_turn_left();
  virtual void calculate_turn_right();

  virtual int activate_weapon();
  virtual int activate_special();
};

class OrzMissile : public Missile {
  public:
  OrzMissile(double oangle, double ov, int odamage, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite);
};

class OrzMarine : public SpaceObject {
  double accel_rate;
  double speed_max;
  int    hotspot_rate;
  int    hotspot_frame;
  int    armour;
  int    invading;
  int    returning;
  int    slot;
  int    damage_frame;
  Ship  *ship;

  public:
  OrzMarine(double ox, double oy, Ship *oship, double oAccelRate,
    double oSpeedMax, int oArmour, int oHotspotRate, int oSlot,
    SpaceSprite *osprite);

  void calculate();
  void animate(Frame *space, Frame *radar);

  void inflict_damage(SpaceObject *other);
  void handle_damage();
};

OrzNemesis::OrzNemesis(double ox, double oy, double shipAngle,
  int team, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, team, shipRadarColor, shipData, cntrl)
{
  int i;

	collide_flag_ally = bit(LAYER_SHIPS);
  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialAccelRate   = get_config_float("Special", "AccelRate", 0) / 100.0;
  specialSpeedMax    = get_config_float("Special", "SpeedMax", 0) / VELOCITY_RATIO;
  specialHotspotRate = get_config_int("Special", "HotspotRate", 0);
  specialArmour      = get_config_int("Special", "Armour", 0);

  scale_frame_rate(&specialHotspotRate);

  for(i = 0; i < MAX_MARINES; i++)
    marine[i] = NULL;

  turretAngle = 0.0;
}

void OrzNemesis::calculate_turn_left()
{
  if(!fire_special)
    Ship::calculate_turn_left();
}

void OrzNemesis::calculate_turn_right()
{
  if(!fire_special)
    Ship::calculate_turn_right();
}

int OrzNemesis::activate_weapon()
{
  if(fire_special)
    return(FALSE);
  spaceFrame->addSpaceObject(new OrzMissile(
    floor((angle + turretAngle) / 5.625) * 5.625, weaponVelocity,
    weaponDamage, weaponRange, weaponArmour, this, data->spriteWeapon));
  return(TRUE);
}

int OrzNemesis::activate_special()
{
  if(turn_left)
    turretAngle -= 1.0;
  if(turn_right)
    turretAngle += 1.0;

  if((fire_weapon) && (crew > 1)) {
    int i = 0;

    while((marine[i] != NULL) && (i < MAX_MARINES))
      i++;
    if(i < MAX_MARINES) {
      marine[i] = new OrzMarine(x, y, this, specialAccelRate,
        specialSpeedMax, specialHotspotRate, specialArmour, i, data->spriteSpecial);
      spaceFrame->addSpaceObject(marine[i]);
      crew--;
      return(TRUE);
    }
  }

  return(FALSE);
}

void OrzNemesis::calculate()
{
  int i;

  for(i = 0; i < MAX_MARINES; i++)
    if((marine[i]) && (!marine[i]->exists()))
      marine[i] = NULL;

  Ship::calculate();
}

void OrzNemesis::animate(Frame *space, Frame *radar)
{
  int turret_index = (int)((angle + turretAngle) / 5.625) + 16;
	turret_index &= 63;

  Ship::animate(space, radar);
  if(phase_count == PHASE_MAX)
    data->spriteExtra->animate((int)(corner_x()), (int)(corner_y()),
      turret_index, space);
}

OrzMissile::OrzMissile(double oangle, double ov, int odamage, double orange,
  int oarmour, Ship *oship, SpaceSprite *osprite) :
  Missile(0.0, 0.0, oangle, ov, odamage, orange, oarmour, oship, oship,
    osprite)
{
  x += cos(angle * ANGLE_RATIO) * 30.0;
  y += sin(angle * ANGLE_RATIO) * 30.0;

  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 20;
  explosionFrameSize  = 4;

  spaceFrame->addSpaceObject(new Animation(x, y,
    ship->data->spriteExtraExplosion, 20, 2, LAYER_EXPLOSIONS));
}

OrzMarine::OrzMarine(double ox, double oy, Ship *oship, double oAccelRate,
  double oSpeedMax, int oHotspotRate, int oArmour, int oSlot,
  SpaceSprite *osprite) :
  SpaceObject(ox, oy, 0.0, oship->allyFlag(), osprite),
  accel_rate(oAccelRate),
  speed_max(oSpeedMax),
  hotspot_rate(oHotspotRate),
  hotspot_frame(0),
  armour(oArmour),
  invading(FALSE),
  returning(FALSE),
  slot(oSlot),
  damage_frame(-1),
  ship(oship)
{
		layer = LAYER_SHIPS;
}

void OrzMarine::calculate()
{
  double v, r, alpha;
  int    chance;

  if((ship) && (!ship->exists())) {
    if((ship->target) && (invading)) {
      blit(ship->target->data->bitmapPanel[0], ship->target->panelBitmap[0],
        14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6),
        14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6), 12, 12);
      ship->target->update_panel = TRUE;
    }
    ship = NULL;
    state = 0;
    return;
  }

  if(invading) {
    if((ship) && (ship->target)) {
      x = ship->target->normal_x();
      y = ship->target->normal_y();

      if(damage_frame != -1) {
        if(damage_frame > 0)
          damage_frame--;
        if(damage_frame == 0) {
          damage_frame = -1;
          draw_sprite(ship->target->panelBitmap[0],
            sprite->get_bitmap(0), 14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6));
          ship->target->update_panel = TRUE;
        }
      }

      chance = random() % 1000;
      if(chance < 10) {
        ship->target->directDamage++;
        tw_sample(ship->data->sampleExtra[0]);
        ship->target->handle_damage();
        damage_frame = 5;
        draw_sprite(ship->target->panelBitmap[0],
          sprite->get_bitmap(1), 14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6));
        ship->target->update_panel = TRUE;
      } else
      if(chance == 10) {
        state = 0;
        play_sample(ship->data->sampleExtra[2], 255, 128, 1000, FALSE);
        blit(ship->target->data->bitmapPanel[0], ship->target->panelBitmap[0],
          14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6),
          14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6), 12, 12);
        ship->target->update_panel = TRUE;
        return;
      }
    } else {
      invading = FALSE;
      returning = TRUE;
      sprite_index = 1;
    }
    return;
  }

  if((ship) && (returning)) {
    angle = trajectory_angle(ship);
		collide_flag_ally = bit(LAYER_SHIPS);
		collide_flag_anyone = 0xff;
		}
  else {
    if((ship) && (ship->target) && (!ship->target->isAlly(ship)))
      angle = trajectory_angle(ship->target);
		}

  alpha = atan2(vy + driftvy, vx + driftvx);
  if((vx + driftvx) < 0.0)
    alpha += PI;
  alpha = ((alpha + PI) / (PI * 2.0)) * 360.0;
  alpha = normalize(alpha, 360.0);

  v = sqrt((vx * vx) + (vy * vy));

  if((fabs(min_delta(alpha, angle, 360.0)) > 0.5) || (v < speed_max)) {
    vx += (cos(angle * ANGLE_RATIO) * accel_rate);
    vy += (sin(angle * ANGLE_RATIO) * accel_rate);

    v = sqrt((vx * vx) + (vy * vy));

    if((v > speed_max) && (v != 0.0)) {
      vx = vx * speed_max / v;
      vy = vy * speed_max / v;
    }

    driftvx *= 0.99;
    driftvy *= 0.99;

    if(hotspot_frame == 0) {
      spaceFrame->addSpaceObject(new Animation(
        normal_x() - (cos(angle * ANGLE_RATIO) * w / 2.5),
        normal_y() - (sin(angle * ANGLE_RATIO) * h / 2.5), hotspotSprite,
        HOTSPOT_FRAMES, 5, LAYER_HOTSPOTS));
      hotspot_frame = hotspot_rate;
    }
  }

  if(hotspot_frame > 0)
    hotspot_frame--;

  r = sqrt((normal_x() - spacePlanet->normal_x()) *
           (normal_x() - spacePlanet->normal_x()) +
           (normal_y() - spacePlanet->normal_y()) *
           (normal_y() - spacePlanet->normal_y()));

  if(r < GRAVITY_RANGE) {
    // calculate gravity angle
    alpha = atan((normal_y() - spacePlanet->normal_y()) /
      (normal_x() - spacePlanet->normal_x()));
    if(normal_x() > spacePlanet->normal_x())
      alpha += PI;

    // apply gravity
    driftvx += (cos(alpha) * (GRAVITY_FORCE / (r * r)));
    driftvy += (sin(alpha) * (GRAVITY_FORCE / (r * r)));
  }

  SpaceObject::calculate();
}

void OrzMarine::animate(Frame *space, Frame *radar)
{
  if(invading)
    return;

  SpaceObject::animate(space, radar);
}

void OrzMarine::inflict_damage(SpaceObject *other)
{
  if((ship) && (ship->target) && (other == ship->target) && (!returning)) {
    invading = TRUE;
    collide_flag_anyone = 0;
    play_sample(ship->data->sampleExtra[1], 255, 128, 1000, FALSE);
    draw_sprite(ship->target->panelBitmap[0],
      sprite->get_bitmap(0), 14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6));
    ship->target->update_panel = TRUE;
  }
  if((ship) && (other == ship) && (returning)) {
    state = 0;
    ship->repair++;
    ship->handle_damage();
  }
}

void OrzMarine::handle_damage()
{
  if((damage > 0) || (directDamage > 0)) {
    armour -= damage;
    armour -= directDamage;
    damage = 0;
    directDamage = 0;
    if(armour <= 0)
      state = 0;
  }
}

Ship *getOrzNemesis(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new OrzNemesis(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO OrzNemesisInfo =
{
  "orzne",
  "Orz Nemesis",
  getOrzNemesis
};
