
#include "ship.h"

class PkunkFury : public Ship {
  int reborn;

  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  public:
  PkunkFury(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual void handle_damage();
  virtual void calculate_phase();
  virtual int activate_weapon();
  virtual void calculate_fire_special();
};

PkunkFury::PkunkFury(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl),
  reborn(FALSE)
{

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponDamage   = get_config_int("Weapon", "Armour", 0);
}

void PkunkFury::handle_damage()
{
  Ship::handle_damage();

  if(state)
    return;

  double rx, ry;

  if((random() % 2) == 0) {
    rx = (double)(random() % (int)(X_MAX));
    ry = (double)(random() % (int)(Y_MAX));
    if(distance(spacePlanet) < 1000.0) {
      rx += cos(trajectory_angle(spacePlanet) * ANGLE_RATIO) * 1000.0;
      ry += sin(trajectory_angle(spacePlanet) * ANGLE_RATIO) * 1000.0;
    }

    meleeVictory = NULL;
    state = 1;
    x = rx;
    y = ry;
	driftvx = vx = driftvy = vy = 0.0;
    crew = crew_max;
    batt = batt_max;
    phase_count = 0;
    reborn = TRUE;
    update_panel = TRUE;
    tw_sample(data->sampleExtra[0]);
  }
}

void PkunkFury::calculate_phase()
{
  if(phase_count < PHASE_MAX) {
    if(phase_step > 0)
      phase_step--;
    else {
      phase_step = PHASE_DELAY;
      spaceFrame->addSpaceObject(new Animation(
        x - (cos(angle * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
        y - (sin(angle * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
        phaseSprite, HOT_COLORS, 5, LAYER_SHIPS));
      if(reborn) {
        spaceFrame->addSpaceObject(new Animation(
          x - (cos((angle + 90.0) * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
          y - (sin((angle + 90.0) * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
          phaseSprite, HOT_COLORS, 5, LAYER_SHIPS));
        spaceFrame->addSpaceObject(new Animation(
          x - (cos((angle + 180.0) * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
          y - (sin((angle + 180.0) * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
          phaseSprite, HOT_COLORS, 5, LAYER_SHIPS));
        spaceFrame->addSpaceObject(new Animation(
          x - (cos((angle + 270.0) * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
          y - (sin((angle + 270.0) * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
          phaseSprite, HOT_COLORS, 5, LAYER_SHIPS));
      }
      phase_count++;
    }
  }
}

int PkunkFury::activate_weapon()
{
  spaceFrame->addSpaceObject(new AnimatedShot(
    0.0, -(height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, this, data->spriteWeapon, 10, 1));
  spaceFrame->addSpaceObject(new AnimatedShot(
    -(width() / 2.0), 0.0, angle - 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, this, data->spriteWeapon, 10, 1));
  spaceFrame->addSpaceObject(new AnimatedShot(
    width() / 2.0, 0.0, angle + 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, this, data->spriteWeapon, 10, 1));

  return(TRUE);
}

void PkunkFury::calculate_fire_special()
{
  int i;

  if(fire_special) {
    if((special_recharge != 0) || (batt >= batt_max))
      return;

    batt += special_drain;
    if(batt > batt_max)
      batt = batt_max;

    special_recharge = special_rate;

    stop_sample(data->sampleSpecial[special_sample]);

    do {
      i = random() % 6;
    } while(i == special_sample);
    special_sample = i;

    tw_sample(data->sampleSpecial[special_sample]);
  }
}

Ship *getPkunkFury(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new PkunkFury(sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO PkunkFuryInfo =
{
  "pkufu",
  "Pkunk Fury",
  getPkunkFury
};
