
#include "ship.h"

#include "../frame.h"

class YehatTerminator : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  int          specialFrames;
  int          shieldFrames;

  public:
  YehatTerminator(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
  virtual void animate(Frame *space, Frame *radar);
  virtual void handle_damage();
};

YehatTerminator::YehatTerminator(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialFrames = get_config_int("Special", "Frames", 0);
  shieldFrames  = 0;
}

int YehatTerminator::activate_weapon()
{
  spaceFrame->addSpaceObject(new Missile(-24.0, -14,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteWeapon));
  spaceFrame->addSpaceObject(new Missile(24.0, -14,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteWeapon));
  return(TRUE);
}

int YehatTerminator::activate_special()
{
  shieldFrames = specialFrames;
  return(TRUE);
}

void YehatTerminator::calculate()
{
   Ship::calculate();
   if(shieldFrames > 0)
     shieldFrames--;
}

void YehatTerminator::animate(Frame *space, Frame *radar)
{
  if(phase_count < PHASE_MAX)
    return;
  if(shieldFrames != 0)
    data->spriteSpecial->animate((int)(corner_x()), (int)(corner_y()), sprite_index, space);
  else
    sprite->animate((int)(corner_x()), (int)(corner_y()), sprite_index, space);
  if(radar_color >= 0) {
    putpixel(radar->frame, (int)(normal_x() / (X_MAX / 56.0)), (int)(normal_y() / (Y_MAX / 48.0)), pallete_color[radar_color]);
    radar->add_box(
      (int)(normal_x() / (X_MAX / 56.0)),
      (int)(normal_y() / (Y_MAX / 48.0)), 1, 1);
  }
}

void YehatTerminator::handle_damage()
{
  if(shieldFrames != 0)
    damage = 0;
  Ship::handle_damage();
}

Ship *getYehatTerminator(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new YehatTerminator(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO YehatTerminatorInfo =
{
  "yehte",
  "Yehat Terminator",
  getYehatTerminator
};
