
#include "ship.h"

class NaroolLurker : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  int cloak;
  int cloak_frame;

  public:
	static int cloak_color[3];
  NaroolLurker(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int isAlly(SpaceLocation *other);
  virtual int activate_weapon();
  virtual void calculate_fire_special();
  virtual void calculate_hotspots();
  virtual void calculate();
  virtual void animate(Frame *space, Frame *radar);
};
int NaroolLurker::cloak_color[3] = { 15, 11, 9 };

NaroolLurker::NaroolLurker(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{
  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  cloak = FALSE;
  cloak_frame = 0;
}

int NaroolLurker::isAlly(SpaceLocation *other)
{
  if(cloak_frame == 30)
    return(TRUE);
  return(SpaceLocation::isAlly(other));
}

int NaroolLurker::activate_weapon()
{
	double a;
	if (target) {
		double r = distance(target);
		r = r / weaponVelocity;
		double x = min_delta( normal_x() , target->normal_x() - 
				(get_vx() - target->get_vx()) * r, X_MAX);
		double y = min_delta( normal_y() , target->normal_y() - 
				(get_vy() - target->get_vy()) * r, Y_MAX);
		a = atan2(y, x) * 180 / PI - 180;
		}
	else a = angle;
  spaceFrame->addSpaceObject(new AnimatedShot(0.0, 0.0,
    a , weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    this, data->spriteWeapon, 50, 1));
  return(TRUE);
}

void NaroolLurker::calculate_fire_special()
{
  special_low = FALSE;

  if(fire_special) {
    if((batt < special_drain) && (!cloak)) {
      special_low = TRUE;
      return;
    }

    if(special_recharge != 0)
      return;

    if(cloak) {
      cloak = FALSE;

      stop_sample(data->sampleSpecial[1]);
      tw_sample(data->sampleSpecial[1]);
    } else {
      cloak = TRUE;
      stop_sample(data->sampleSpecial[0]);
      tw_sample(data->sampleSpecial[0]);
      batt -= special_drain;
    }

    special_recharge = special_rate;
  }
}

void NaroolLurker::calculate_hotspots()
{
  if(!cloak)
    Ship::calculate_hotspots();
}

void NaroolLurker::calculate()
{
  if(cloak && (recharge_step == 0)) batt -= recharge_amount - 1;
  if((cloak) && (cloak_frame < 30))
    cloak_frame++;
  if((!cloak) && (cloak_frame > 0))
    cloak_frame--;

  Ship::calculate();
}

void NaroolLurker::animate(Frame *space, Frame *radar)
{
  if(phase_count < PHASE_MAX)
    return;

  if((cloak_frame > 0) && (cloak_frame < 30))
    sprite->animate_character((int)(corner_x()), (int)(corner_y()),
      sprite_index, pallete_color[cloak_color[cloak_frame / 10]], space);
  else
  if((cloak_frame == 30))
    sprite->animate_character((int)(corner_x()), (int)(corner_y()),
      sprite_index, pallete_color[0], space);
  else
    Ship::animate(space, radar);
}

Ship *getNaroolLurker(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new NaroolLurker(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO NaroolLurkerInfo =
{
  "narlu",
  "Narool Lurker",
  getNaroolLurker
};
