
#include "ship.h"

class TerranDestroyer : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialRange;
  double       specialVelocity;
  int          specialDamage;
  int          specialArmour;

  public:
  TerranDestroyer(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
};

class TerranMissile : public Missile {
  public:
  TerranMissile(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);
};

class TerranMiniMissile : public Missile {
  public:
  TerranMiniMissile(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);
};

TerranDestroyer::TerranDestroyer(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl) {

  weaponRange    = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponVelocity = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialRange    = (double)(get_config_int("Special", "Range", 0)) * RANGE_RATIO;
  specialVelocity = (double)(get_config_int("Special", "Velocity", 0)) / VELOCITY_RATIO;
  specialDamage   = get_config_int("Special", "Damage", 0);
  specialArmour   = get_config_int("Special", "Armour", 0);

}

static const int trate = 0;

int TerranDestroyer::activate_weapon() {
  spaceFrame->addSpaceObject(new TerranMissile(
    0.0, -(height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, trate, this, data->spriteWeapon));
  return(TRUE);
}

int TerranDestroyer::activate_special() {
  spaceFrame->addSpaceObject(new TerranMiniMissile(
    0.0, -(height() / 2.0), angle - 15.0 + ((double)(random() % 3) * 10.0), specialVelocity,
    specialDamage, specialRange, specialArmour, trate, this, data->spriteWeapon));
  return(TRUE);
}

TerranMissile::TerranMissile(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, double otrate,
  Ship *oship, SpaceSprite *osprite) :
//  Missile(ox, oy, oangle, ov, odamage, orange, oarmour, otrate, oship,
  Missile(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship,
    osprite)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 10;
  explosionFrameSize  = 5;
}

TerranMiniMissile::TerranMiniMissile(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, double otrate,
  Ship *oship, SpaceSprite *osprite) :
//  Missile(ox, oy, oangle, ov, odamage, orange, oarmour, otrate, oship,
  Missile(ox, oy, oangle, ov, odamage, orange, oarmour,  oship, oship,
    osprite)
{
}

Ship *getTerranDestroyer(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new TerranDestroyer(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO TerranDestroyerInfo =
{
  "terds",
  "Terran Destroyer",
  getTerranDestroyer
};
