
#include "ship.h"

class VirtaoHost : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponTurnRate;

  double specialRange;
  double specialPower;

  public:
  VirtaoHost(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
};

class VirtaoMissile : public HomingMissile {
  public:
  VirtaoMissile(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);
};

VirtaoHost::VirtaoHost(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl) {

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponTurnRate = get_config_float("Weapon", "TurnRate", 0);

  specialRange  = get_config_float("Special", "Range", 0) * RANGE_RATIO;
  specialPower  = get_config_float("Special", "Power", 0) * RANGE_RATIO;

}

int VirtaoHost::activate_weapon() {
  spaceFrame->addSpaceObject(new VirtaoMissile(
    0.0, -(height() * 1.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, weaponTurnRate, this, data->spriteWeapon));
  return(TRUE);
}

int VirtaoHost::activate_special() {
  int fire = FALSE, i;
  SpaceObject *o;

  for(i = 0; i < SPACE_LAYERS; i++) {
    o = spaceFrame->getFirstObject(i);
    while(o != NULL) {
			if( (!o->isAlly(this)) && (distance(o) < specialRange) &&
					canCollide(o) && o->canCollide(this)) {
				double a = trajectory_angle(o) * PI / 180;
				o->translate(cos(a) * 300, sin(a) * specialPower);
				fire = TRUE;
				}
			o = o->next;
			}
		}
	if (fire) tw_sample((SAMPLE *)(melee[MELEE_BOOM + 0].dat));

	return(fire);
	}

VirtaoMissile::VirtaoMissile(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, double otrate,
  Ship *oship, SpaceSprite *osprite) :
  HomingMissile(ox, oy, oangle, ov, odamage, orange, oarmour, otrate, oship,
    osprite)
{
	collide_flag_ally = bit(LAYER_SHIPS) | bit(LAYER_SHOTS);
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 10;
  explosionFrameSize  = 5;
}

Ship *getVirtaoHost(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new VirtaoHost(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO VirtaoHostInfo =
{
  "virli",
  "Virtao Host",
  getVirtaoHost
};
